/* valaccodemethodcallmodule.vala
 *
 * Copyright (C) 2006-2009  Jürg Billeter
 * Copyright (C) 2006-2008  Raffaele Sandrini
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.

 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.

 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA
 *
 * Author:
 * 	Jürg Billeter <j@bitron.ch>
 *	Raffaele Sandrini <raffaele@sandrini.ch>
 */

#include <glib.h>
#include <glib-object.h>
#include <vala.h>
#include <stdlib.h>
#include <string.h>
#include <valaccode.h>
#include <gee.h>
#include <float.h>
#include <math.h>


#define VALA_TYPE_CCODE_MODULE (vala_ccode_module_get_type ())
#define VALA_CCODE_MODULE(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), VALA_TYPE_CCODE_MODULE, ValaCCodeModule))
#define VALA_CCODE_MODULE_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), VALA_TYPE_CCODE_MODULE, ValaCCodeModuleClass))
#define VALA_IS_CCODE_MODULE(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), VALA_TYPE_CCODE_MODULE))
#define VALA_IS_CCODE_MODULE_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), VALA_TYPE_CCODE_MODULE))
#define VALA_CCODE_MODULE_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), VALA_TYPE_CCODE_MODULE, ValaCCodeModuleClass))

typedef struct _ValaCCodeModule ValaCCodeModule;
typedef struct _ValaCCodeModuleClass ValaCCodeModuleClass;
typedef struct _ValaCCodeModulePrivate ValaCCodeModulePrivate;

#define VALA_TYPE_CCODE_BASE_MODULE (vala_ccode_base_module_get_type ())
#define VALA_CCODE_BASE_MODULE(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), VALA_TYPE_CCODE_BASE_MODULE, ValaCCodeBaseModule))
#define VALA_CCODE_BASE_MODULE_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), VALA_TYPE_CCODE_BASE_MODULE, ValaCCodeBaseModuleClass))
#define VALA_IS_CCODE_BASE_MODULE(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), VALA_TYPE_CCODE_BASE_MODULE))
#define VALA_IS_CCODE_BASE_MODULE_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), VALA_TYPE_CCODE_BASE_MODULE))
#define VALA_CCODE_BASE_MODULE_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), VALA_TYPE_CCODE_BASE_MODULE, ValaCCodeBaseModuleClass))

typedef struct _ValaCCodeBaseModule ValaCCodeBaseModule;
typedef struct _ValaCCodeBaseModuleClass ValaCCodeBaseModuleClass;
typedef struct _ValaCCodeBaseModulePrivate ValaCCodeBaseModulePrivate;

#define VALA_TYPE_CCODE_DECLARATION_SPACE (vala_ccode_declaration_space_get_type ())
#define VALA_CCODE_DECLARATION_SPACE(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), VALA_TYPE_CCODE_DECLARATION_SPACE, ValaCCodeDeclarationSpace))
#define VALA_CCODE_DECLARATION_SPACE_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), VALA_TYPE_CCODE_DECLARATION_SPACE, ValaCCodeDeclarationSpaceClass))
#define VALA_IS_CCODE_DECLARATION_SPACE(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), VALA_TYPE_CCODE_DECLARATION_SPACE))
#define VALA_IS_CCODE_DECLARATION_SPACE_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), VALA_TYPE_CCODE_DECLARATION_SPACE))
#define VALA_CCODE_DECLARATION_SPACE_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), VALA_TYPE_CCODE_DECLARATION_SPACE, ValaCCodeDeclarationSpaceClass))

typedef struct _ValaCCodeDeclarationSpace ValaCCodeDeclarationSpace;
typedef struct _ValaCCodeDeclarationSpaceClass ValaCCodeDeclarationSpaceClass;

#define VALA_TYPE_CCODE_STRUCT_MODULE (vala_ccode_struct_module_get_type ())
#define VALA_CCODE_STRUCT_MODULE(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), VALA_TYPE_CCODE_STRUCT_MODULE, ValaCCodeStructModule))
#define VALA_CCODE_STRUCT_MODULE_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), VALA_TYPE_CCODE_STRUCT_MODULE, ValaCCodeStructModuleClass))
#define VALA_IS_CCODE_STRUCT_MODULE(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), VALA_TYPE_CCODE_STRUCT_MODULE))
#define VALA_IS_CCODE_STRUCT_MODULE_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), VALA_TYPE_CCODE_STRUCT_MODULE))
#define VALA_CCODE_STRUCT_MODULE_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), VALA_TYPE_CCODE_STRUCT_MODULE, ValaCCodeStructModuleClass))

typedef struct _ValaCCodeStructModule ValaCCodeStructModule;
typedef struct _ValaCCodeStructModuleClass ValaCCodeStructModuleClass;
typedef struct _ValaCCodeStructModulePrivate ValaCCodeStructModulePrivate;

#define VALA_TYPE_CCODE_METHOD_MODULE (vala_ccode_method_module_get_type ())
#define VALA_CCODE_METHOD_MODULE(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), VALA_TYPE_CCODE_METHOD_MODULE, ValaCCodeMethodModule))
#define VALA_CCODE_METHOD_MODULE_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), VALA_TYPE_CCODE_METHOD_MODULE, ValaCCodeMethodModuleClass))
#define VALA_IS_CCODE_METHOD_MODULE(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), VALA_TYPE_CCODE_METHOD_MODULE))
#define VALA_IS_CCODE_METHOD_MODULE_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), VALA_TYPE_CCODE_METHOD_MODULE))
#define VALA_CCODE_METHOD_MODULE_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), VALA_TYPE_CCODE_METHOD_MODULE, ValaCCodeMethodModuleClass))

typedef struct _ValaCCodeMethodModule ValaCCodeMethodModule;
typedef struct _ValaCCodeMethodModuleClass ValaCCodeMethodModuleClass;
typedef struct _ValaCCodeMethodModulePrivate ValaCCodeMethodModulePrivate;

#define VALA_TYPE_CCODE_CONTROL_FLOW_MODULE (vala_ccode_control_flow_module_get_type ())
#define VALA_CCODE_CONTROL_FLOW_MODULE(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), VALA_TYPE_CCODE_CONTROL_FLOW_MODULE, ValaCCodeControlFlowModule))
#define VALA_CCODE_CONTROL_FLOW_MODULE_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), VALA_TYPE_CCODE_CONTROL_FLOW_MODULE, ValaCCodeControlFlowModuleClass))
#define VALA_IS_CCODE_CONTROL_FLOW_MODULE(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), VALA_TYPE_CCODE_CONTROL_FLOW_MODULE))
#define VALA_IS_CCODE_CONTROL_FLOW_MODULE_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), VALA_TYPE_CCODE_CONTROL_FLOW_MODULE))
#define VALA_CCODE_CONTROL_FLOW_MODULE_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), VALA_TYPE_CCODE_CONTROL_FLOW_MODULE, ValaCCodeControlFlowModuleClass))

typedef struct _ValaCCodeControlFlowModule ValaCCodeControlFlowModule;
typedef struct _ValaCCodeControlFlowModuleClass ValaCCodeControlFlowModuleClass;
typedef struct _ValaCCodeControlFlowModulePrivate ValaCCodeControlFlowModulePrivate;

#define VALA_TYPE_CCODE_MEMBER_ACCESS_MODULE (vala_ccode_member_access_module_get_type ())
#define VALA_CCODE_MEMBER_ACCESS_MODULE(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), VALA_TYPE_CCODE_MEMBER_ACCESS_MODULE, ValaCCodeMemberAccessModule))
#define VALA_CCODE_MEMBER_ACCESS_MODULE_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), VALA_TYPE_CCODE_MEMBER_ACCESS_MODULE, ValaCCodeMemberAccessModuleClass))
#define VALA_IS_CCODE_MEMBER_ACCESS_MODULE(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), VALA_TYPE_CCODE_MEMBER_ACCESS_MODULE))
#define VALA_IS_CCODE_MEMBER_ACCESS_MODULE_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), VALA_TYPE_CCODE_MEMBER_ACCESS_MODULE))
#define VALA_CCODE_MEMBER_ACCESS_MODULE_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), VALA_TYPE_CCODE_MEMBER_ACCESS_MODULE, ValaCCodeMemberAccessModuleClass))

typedef struct _ValaCCodeMemberAccessModule ValaCCodeMemberAccessModule;
typedef struct _ValaCCodeMemberAccessModuleClass ValaCCodeMemberAccessModuleClass;
typedef struct _ValaCCodeMemberAccessModulePrivate ValaCCodeMemberAccessModulePrivate;

#define VALA_TYPE_CCODE_ASSIGNMENT_MODULE (vala_ccode_assignment_module_get_type ())
#define VALA_CCODE_ASSIGNMENT_MODULE(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), VALA_TYPE_CCODE_ASSIGNMENT_MODULE, ValaCCodeAssignmentModule))
#define VALA_CCODE_ASSIGNMENT_MODULE_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), VALA_TYPE_CCODE_ASSIGNMENT_MODULE, ValaCCodeAssignmentModuleClass))
#define VALA_IS_CCODE_ASSIGNMENT_MODULE(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), VALA_TYPE_CCODE_ASSIGNMENT_MODULE))
#define VALA_IS_CCODE_ASSIGNMENT_MODULE_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), VALA_TYPE_CCODE_ASSIGNMENT_MODULE))
#define VALA_CCODE_ASSIGNMENT_MODULE_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), VALA_TYPE_CCODE_ASSIGNMENT_MODULE, ValaCCodeAssignmentModuleClass))

typedef struct _ValaCCodeAssignmentModule ValaCCodeAssignmentModule;
typedef struct _ValaCCodeAssignmentModuleClass ValaCCodeAssignmentModuleClass;
typedef struct _ValaCCodeAssignmentModulePrivate ValaCCodeAssignmentModulePrivate;

#define VALA_TYPE_CCODE_METHOD_CALL_MODULE (vala_ccode_method_call_module_get_type ())
#define VALA_CCODE_METHOD_CALL_MODULE(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), VALA_TYPE_CCODE_METHOD_CALL_MODULE, ValaCCodeMethodCallModule))
#define VALA_CCODE_METHOD_CALL_MODULE_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), VALA_TYPE_CCODE_METHOD_CALL_MODULE, ValaCCodeMethodCallModuleClass))
#define VALA_IS_CCODE_METHOD_CALL_MODULE(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), VALA_TYPE_CCODE_METHOD_CALL_MODULE))
#define VALA_IS_CCODE_METHOD_CALL_MODULE_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), VALA_TYPE_CCODE_METHOD_CALL_MODULE))
#define VALA_CCODE_METHOD_CALL_MODULE_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), VALA_TYPE_CCODE_METHOD_CALL_MODULE, ValaCCodeMethodCallModuleClass))

typedef struct _ValaCCodeMethodCallModule ValaCCodeMethodCallModule;
typedef struct _ValaCCodeMethodCallModuleClass ValaCCodeMethodCallModuleClass;
typedef struct _ValaCCodeMethodCallModulePrivate ValaCCodeMethodCallModulePrivate;

#define VALA_TYPE_CCODE_GENERATOR (vala_ccode_generator_get_type ())
#define VALA_CCODE_GENERATOR(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), VALA_TYPE_CCODE_GENERATOR, ValaCCodeGenerator))
#define VALA_CCODE_GENERATOR_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), VALA_TYPE_CCODE_GENERATOR, ValaCCodeGeneratorClass))
#define VALA_IS_CCODE_GENERATOR(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), VALA_TYPE_CCODE_GENERATOR))
#define VALA_IS_CCODE_GENERATOR_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), VALA_TYPE_CCODE_GENERATOR))
#define VALA_CCODE_GENERATOR_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), VALA_TYPE_CCODE_GENERATOR, ValaCCodeGeneratorClass))

typedef struct _ValaCCodeGenerator ValaCCodeGenerator;
typedef struct _ValaCCodeGeneratorClass ValaCCodeGeneratorClass;

/**
 * Code visitor generating C Code.
 */
struct _ValaCCodeModule {
	GTypeInstance parent_instance;
	volatile int ref_count;
	ValaCCodeModulePrivate * priv;
};

struct _ValaCCodeModuleClass {
	GTypeClass parent_class;
	void (*finalize) (ValaCCodeModule *self);
	void (*emit) (ValaCCodeModule* self, ValaCodeContext* context);
	void (*visit_source_file) (ValaCCodeModule* self, ValaSourceFile* source_file);
	void (*visit_class) (ValaCCodeModule* self, ValaClass* cl);
	void (*visit_interface) (ValaCCodeModule* self, ValaInterface* iface);
	void (*visit_struct) (ValaCCodeModule* self, ValaStruct* st);
	void (*visit_enum) (ValaCCodeModule* self, ValaEnum* en);
	void (*visit_error_domain) (ValaCCodeModule* self, ValaErrorDomain* edomain);
	void (*visit_delegate) (ValaCCodeModule* self, ValaDelegate* d);
	void (*visit_member) (ValaCCodeModule* self, ValaMember* m);
	void (*visit_constant) (ValaCCodeModule* self, ValaConstant* c);
	void (*visit_field) (ValaCCodeModule* self, ValaField* f);
	void (*visit_method) (ValaCCodeModule* self, ValaMethod* m);
	void (*visit_creation_method) (ValaCCodeModule* self, ValaCreationMethod* m);
	void (*visit_formal_parameter) (ValaCCodeModule* self, ValaFormalParameter* p);
	void (*visit_property) (ValaCCodeModule* self, ValaProperty* prop);
	void (*visit_property_accessor) (ValaCCodeModule* self, ValaPropertyAccessor* acc);
	void (*visit_signal) (ValaCCodeModule* self, ValaSignal* sig);
	void (*visit_constructor) (ValaCCodeModule* self, ValaConstructor* c);
	void (*visit_destructor) (ValaCCodeModule* self, ValaDestructor* d);
	void (*visit_block) (ValaCCodeModule* self, ValaBlock* b);
	void (*visit_empty_statement) (ValaCCodeModule* self, ValaEmptyStatement* stmt);
	void (*visit_declaration_statement) (ValaCCodeModule* self, ValaDeclarationStatement* stmt);
	void (*visit_local_variable) (ValaCCodeModule* self, ValaLocalVariable* local);
	void (*visit_initializer_list) (ValaCCodeModule* self, ValaInitializerList* list);
	void (*visit_end_full_expression) (ValaCCodeModule* self, ValaExpression* expr);
	void (*visit_expression_statement) (ValaCCodeModule* self, ValaExpressionStatement* stmt);
	void (*visit_if_statement) (ValaCCodeModule* self, ValaIfStatement* stmt);
	void (*visit_switch_statement) (ValaCCodeModule* self, ValaSwitchStatement* stmt);
	void (*visit_switch_section) (ValaCCodeModule* self, ValaSwitchSection* section);
	void (*visit_switch_label) (ValaCCodeModule* self, ValaSwitchLabel* label);
	void (*visit_loop) (ValaCCodeModule* self, ValaLoop* stmt);
	void (*visit_foreach_statement) (ValaCCodeModule* self, ValaForeachStatement* stmt);
	void (*visit_break_statement) (ValaCCodeModule* self, ValaBreakStatement* stmt);
	void (*visit_continue_statement) (ValaCCodeModule* self, ValaContinueStatement* stmt);
	void (*visit_return_statement) (ValaCCodeModule* self, ValaReturnStatement* stmt);
	void (*visit_yield_statement) (ValaCCodeModule* self, ValaYieldStatement* stmt);
	void (*visit_throw_statement) (ValaCCodeModule* self, ValaThrowStatement* stmt);
	void (*visit_try_statement) (ValaCCodeModule* self, ValaTryStatement* stmt);
	void (*visit_catch_clause) (ValaCCodeModule* self, ValaCatchClause* clause);
	void (*visit_lock_statement) (ValaCCodeModule* self, ValaLockStatement* stmt);
	void (*visit_delete_statement) (ValaCCodeModule* self, ValaDeleteStatement* stmt);
	void (*visit_expression) (ValaCCodeModule* self, ValaExpression* expr);
	void (*visit_array_creation_expression) (ValaCCodeModule* self, ValaArrayCreationExpression* expr);
	void (*visit_boolean_literal) (ValaCCodeModule* self, ValaBooleanLiteral* expr);
	void (*visit_character_literal) (ValaCCodeModule* self, ValaCharacterLiteral* expr);
	void (*visit_integer_literal) (ValaCCodeModule* self, ValaIntegerLiteral* expr);
	void (*visit_real_literal) (ValaCCodeModule* self, ValaRealLiteral* expr);
	void (*visit_string_literal) (ValaCCodeModule* self, ValaStringLiteral* expr);
	void (*visit_null_literal) (ValaCCodeModule* self, ValaNullLiteral* expr);
	void (*visit_member_access) (ValaCCodeModule* self, ValaMemberAccess* expr);
	void (*visit_method_call) (ValaCCodeModule* self, ValaMethodCall* expr);
	void (*visit_element_access) (ValaCCodeModule* self, ValaElementAccess* expr);
	void (*visit_base_access) (ValaCCodeModule* self, ValaBaseAccess* expr);
	void (*visit_postfix_expression) (ValaCCodeModule* self, ValaPostfixExpression* expr);
	void (*visit_object_creation_expression) (ValaCCodeModule* self, ValaObjectCreationExpression* expr);
	void (*visit_sizeof_expression) (ValaCCodeModule* self, ValaSizeofExpression* expr);
	void (*visit_typeof_expression) (ValaCCodeModule* self, ValaTypeofExpression* expr);
	void (*visit_unary_expression) (ValaCCodeModule* self, ValaUnaryExpression* expr);
	void (*visit_cast_expression) (ValaCCodeModule* self, ValaCastExpression* expr);
	void (*visit_pointer_indirection) (ValaCCodeModule* self, ValaPointerIndirection* expr);
	void (*visit_addressof_expression) (ValaCCodeModule* self, ValaAddressofExpression* expr);
	void (*visit_reference_transfer_expression) (ValaCCodeModule* self, ValaReferenceTransferExpression* expr);
	void (*visit_binary_expression) (ValaCCodeModule* self, ValaBinaryExpression* expr);
	void (*visit_type_check) (ValaCCodeModule* self, ValaTypeCheck* expr);
	void (*visit_lambda_expression) (ValaCCodeModule* self, ValaLambdaExpression* l);
	void (*visit_assignment) (ValaCCodeModule* self, ValaAssignment* a);
	char* (*get_custom_creturn_type) (ValaCCodeModule* self, ValaMethod* m);
	void (*generate_dynamic_method_wrapper) (ValaCCodeModule* self, ValaDynamicMethod* method);
	gboolean (*method_has_wrapper) (ValaCCodeModule* self, ValaMethod* method);
	ValaCCodeIdentifier* (*get_value_setter_function) (ValaCCodeModule* self, ValaDataType* type_reference);
	ValaCCodeExpression* (*get_construct_property_assignment) (ValaCCodeModule* self, ValaCCodeConstant* canonical_cconstant, ValaDataType* property_type, ValaCCodeExpression* value);
	ValaCCodeFunctionCall* (*get_param_spec) (ValaCCodeModule* self, ValaProperty* prop);
	ValaCCodeFunctionCall* (*get_signal_creation) (ValaCCodeModule* self, ValaSignal* sig, ValaTypeSymbol* type);
	ValaCCodeFragment* (*register_dbus_info) (ValaCCodeModule* self, ValaObjectTypeSymbol* bindable);
	char* (*get_dynamic_property_getter_cname) (ValaCCodeModule* self, ValaDynamicProperty* node);
	char* (*get_dynamic_property_setter_cname) (ValaCCodeModule* self, ValaDynamicProperty* node);
	char* (*get_dynamic_signal_cname) (ValaCCodeModule* self, ValaDynamicSignal* node);
	char* (*get_dynamic_signal_connect_wrapper_name) (ValaCCodeModule* self, ValaDynamicSignal* node);
	char* (*get_dynamic_signal_disconnect_wrapper_name) (ValaCCodeModule* self, ValaDynamicSignal* node);
	void (*generate_marshaller) (ValaCCodeModule* self, GeeList* params, ValaDataType* return_type, gboolean dbus);
	char* (*get_marshaller_function) (ValaCCodeModule* self, GeeList* params, ValaDataType* return_type, const char* prefix, gboolean dbus);
	char* (*get_array_length_cname) (ValaCCodeModule* self, const char* array_cname, gint dim);
	ValaCCodeExpression* (*get_array_length_cexpression) (ValaCCodeModule* self, ValaExpression* array_expr, gint dim);
	char* (*get_array_size_cname) (ValaCCodeModule* self, const char* array_cname);
	ValaCCodeExpression* (*get_array_size_cexpression) (ValaCCodeModule* self, ValaExpression* array_expr);
	void (*add_simple_check) (ValaCCodeModule* self, ValaCodeNode* node, ValaCCodeFragment* cfrag);
};

/**
 * Code visitor generating C Code.
 */
struct _ValaCCodeBaseModule {
	ValaCCodeModule parent_instance;
	ValaCCodeBaseModulePrivate * priv;
	ValaSymbol* root_symbol;
	ValaSymbol* current_symbol;
	ValaTypeSymbol* current_type_symbol;
	ValaClass* current_class;
	ValaMethod* current_method;
	ValaDataType* current_return_type;
	ValaTryStatement* current_try;
	ValaPropertyAccessor* current_property_accessor;
	ValaCCodeDeclarationSpace* header_declarations;
	ValaCCodeDeclarationSpace* internal_header_declarations;
	ValaCCodeDeclarationSpace* source_declarations;
	ValaCCodeFragment* source_signal_marshaller_declaration;
	ValaCCodeFragment* source_type_member_definition;
	ValaCCodeFragment* class_init_fragment;
	ValaCCodeFragment* base_init_fragment;
	ValaCCodeFragment* class_finalize_fragment;
	ValaCCodeFragment* base_finalize_fragment;
	ValaCCodeFragment* instance_init_fragment;
	ValaCCodeFragment* instance_finalize_fragment;
	ValaCCodeFragment* source_signal_marshaller_definition;
	ValaCCodeFragment* module_init_fragment;
	ValaCCodeStruct* param_spec_struct;
	ValaCCodeStruct* closure_struct;
	ValaCCodeEnum* prop_enum;
	ValaCCodeFunction* function;
	ValaCCodeFragment* pre_statement_fragment;
	GeeArrayList* temp_vars;
	GeeArrayList* temp_ref_vars;
	GeeSet* user_marshal_set;
	GeeSet* predefined_marshal_set;
	gint next_temp_var_id;
	gboolean in_creation_method;
	gboolean in_constructor;
	gboolean in_static_or_class_ctor;
	gboolean current_method_inner_error;
	gint next_coroutine_state;
	ValaDataType* bool_type;
	ValaDataType* char_type;
	ValaDataType* uchar_type;
	ValaDataType* unichar_type;
	ValaDataType* short_type;
	ValaDataType* ushort_type;
	ValaDataType* int_type;
	ValaDataType* uint_type;
	ValaDataType* long_type;
	ValaDataType* ulong_type;
	ValaDataType* int8_type;
	ValaDataType* uint8_type;
	ValaDataType* int16_type;
	ValaDataType* uint16_type;
	ValaDataType* int32_type;
	ValaDataType* uint32_type;
	ValaDataType* int64_type;
	ValaDataType* uint64_type;
	ValaDataType* string_type;
	ValaDataType* float_type;
	ValaDataType* double_type;
	ValaTypeSymbol* gtype_type;
	ValaTypeSymbol* gobject_type;
	ValaErrorType* gerror_type;
	ValaClass* glist_type;
	ValaClass* gslist_type;
	ValaTypeSymbol* gstringbuilder_type;
	ValaTypeSymbol* garray_type;
	ValaTypeSymbol* gbytearray_type;
	ValaTypeSymbol* gptrarray_type;
	ValaDataType* gquark_type;
	ValaStruct* gvalue_type;
	ValaStruct* mutex_type;
	ValaTypeSymbol* type_module_type;
	ValaTypeSymbol* dbus_object_type;
	gboolean in_plugin;
	char* module_init_param_name;
	gboolean gvaluecollector_h_needed;
	gboolean requires_array_free;
	gboolean requires_array_move;
	gboolean requires_array_length;
	gboolean requires_strcmp0;
	gboolean dbus_glib_h_needed;
	gboolean dbus_glib_h_needed_in_header;
	GeeSet* wrappers;
	GeeMap* variable_name_map;
};

struct _ValaCCodeBaseModuleClass {
	ValaCCodeModuleClass parent_class;
	void (*append_vala_array_free) (ValaCCodeBaseModule* self);
	void (*append_vala_array_move) (ValaCCodeBaseModule* self);
	void (*append_vala_array_length) (ValaCCodeBaseModule* self);
	void (*generate_class_struct_declaration) (ValaCCodeBaseModule* self, ValaClass* cl, ValaCCodeDeclarationSpace* decl_space);
	void (*generate_struct_declaration) (ValaCCodeBaseModule* self, ValaStruct* st, ValaCCodeDeclarationSpace* decl_space);
	void (*generate_delegate_declaration) (ValaCCodeBaseModule* self, ValaDelegate* d, ValaCCodeDeclarationSpace* decl_space);
	void (*generate_cparameters) (ValaCCodeBaseModule* self, ValaMethod* m, ValaCCodeDeclarationSpace* decl_space, GeeMap* cparam_map, ValaCCodeFunction* func, ValaCCodeFunctionDeclarator* vdeclarator, GeeMap* carg_map, ValaCCodeFunctionCall* vcall, gint direction);
	ValaCCodeExpression* (*get_dup_func_expression) (ValaCCodeBaseModule* self, ValaDataType* type, ValaSourceReference* source_reference);
	ValaCCodeExpression* (*get_unref_expression) (ValaCCodeBaseModule* self, ValaCCodeExpression* cvar, ValaDataType* type, ValaExpression* expr);
	char* (*get_delegate_target_cname) (ValaCCodeBaseModule* self, const char* delegate_cname);
	ValaCCodeExpression* (*get_delegate_target_cexpression) (ValaCCodeBaseModule* self, ValaExpression* delegate_expr);
	char* (*get_delegate_target_destroy_notify_cname) (ValaCCodeBaseModule* self, const char* delegate_cname);
	ValaCCodeExpression* (*get_ref_cexpression) (ValaCCodeBaseModule* self, ValaDataType* expression_type, ValaCCodeExpression* cexpr, ValaExpression* expr, ValaCodeNode* node);
	void (*generate_class_declaration) (ValaCCodeBaseModule* self, ValaClass* cl, ValaCCodeDeclarationSpace* decl_space);
	void (*generate_interface_declaration) (ValaCCodeBaseModule* self, ValaInterface* iface, ValaCCodeDeclarationSpace* decl_space);
	void (*generate_method_declaration) (ValaCCodeBaseModule* self, ValaMethod* m, ValaCCodeDeclarationSpace* decl_space);
	void (*generate_error_domain_declaration) (ValaCCodeBaseModule* self, ValaErrorDomain* edomain, ValaCCodeDeclarationSpace* decl_space);
	ValaCCodeExpression* (*get_implicit_cast_expression) (ValaCCodeBaseModule* self, ValaCCodeExpression* source_cexpr, ValaDataType* expression_type, ValaDataType* target_type, ValaExpression* expr);
	gboolean (*is_gobject_property) (ValaCCodeBaseModule* self, ValaProperty* prop);
};

struct _ValaCCodeStructModule {
	ValaCCodeBaseModule parent_instance;
	ValaCCodeStructModulePrivate * priv;
};

struct _ValaCCodeStructModuleClass {
	ValaCCodeBaseModuleClass parent_class;
};

/**
 * The link between a method and generated code.
 */
struct _ValaCCodeMethodModule {
	ValaCCodeStructModule parent_instance;
	ValaCCodeMethodModulePrivate * priv;
};

struct _ValaCCodeMethodModuleClass {
	ValaCCodeStructModuleClass parent_class;
	void (*generate_method_result_declaration) (ValaCCodeMethodModule* self, ValaMethod* m, ValaCCodeDeclarationSpace* decl_space, ValaCCodeFunction* cfunc, GeeMap* cparam_map, GeeMap* carg_map);
	void (*generate_parameter) (ValaCCodeMethodModule* self, ValaFormalParameter* param, ValaCCodeDeclarationSpace* decl_space, GeeMap* cparam_map, GeeMap* carg_map);
};

struct _ValaCCodeControlFlowModule {
	ValaCCodeMethodModule parent_instance;
	ValaCCodeControlFlowModulePrivate * priv;
};

struct _ValaCCodeControlFlowModuleClass {
	ValaCCodeMethodModuleClass parent_class;
};

struct _ValaCCodeMemberAccessModule {
	ValaCCodeControlFlowModule parent_instance;
	ValaCCodeMemberAccessModulePrivate * priv;
};

struct _ValaCCodeMemberAccessModuleClass {
	ValaCCodeControlFlowModuleClass parent_class;
};

/**
 * The link between an assignment and generated code.
 */
struct _ValaCCodeAssignmentModule {
	ValaCCodeMemberAccessModule parent_instance;
	ValaCCodeAssignmentModulePrivate * priv;
};

struct _ValaCCodeAssignmentModuleClass {
	ValaCCodeMemberAccessModuleClass parent_class;
};

struct _ValaCCodeMethodCallModule {
	ValaCCodeAssignmentModule parent_instance;
	ValaCCodeMethodCallModulePrivate * priv;
};

struct _ValaCCodeMethodCallModuleClass {
	ValaCCodeAssignmentModuleClass parent_class;
};


static gpointer vala_ccode_method_call_module_parent_class = NULL;

gpointer vala_ccode_module_ref (gpointer instance);
void vala_ccode_module_unref (gpointer instance);
GParamSpec* vala_param_spec_ccode_module (const gchar* name, const gchar* nick, const gchar* blurb, GType object_type, GParamFlags flags);
void vala_value_set_ccode_module (GValue* value, gpointer v_object);
gpointer vala_value_get_ccode_module (const GValue* value);
GType vala_ccode_module_get_type (void);
GType vala_ccode_base_module_get_type (void);
gpointer vala_ccode_declaration_space_ref (gpointer instance);
void vala_ccode_declaration_space_unref (gpointer instance);
GParamSpec* vala_param_spec_ccode_declaration_space (const gchar* name, const gchar* nick, const gchar* blurb, GType object_type, GParamFlags flags);
void vala_value_set_ccode_declaration_space (GValue* value, gpointer v_object);
gpointer vala_value_get_ccode_declaration_space (const GValue* value);
GType vala_ccode_declaration_space_get_type (void);
GType vala_ccode_struct_module_get_type (void);
GType vala_ccode_method_module_get_type (void);
GType vala_ccode_control_flow_module_get_type (void);
GType vala_ccode_member_access_module_get_type (void);
GType vala_ccode_assignment_module_get_type (void);
GType vala_ccode_method_call_module_get_type (void);
enum  {
	VALA_CCODE_METHOD_CALL_MODULE_DUMMY_PROPERTY
};
GType vala_ccode_generator_get_type (void);
ValaCCodeAssignmentModule* vala_ccode_assignment_module_new (ValaCCodeGenerator* codegen, ValaCCodeModule* next);
ValaCCodeAssignmentModule* vala_ccode_assignment_module_construct (GType object_type, ValaCCodeGenerator* codegen, ValaCCodeModule* next);
ValaCCodeMethodCallModule* vala_ccode_method_call_module_new (ValaCCodeGenerator* codegen, ValaCCodeModule* next);
ValaCCodeMethodCallModule* vala_ccode_method_call_module_construct (GType object_type, ValaCCodeGenerator* codegen, ValaCCodeModule* next);
ValaCCodeMethodCallModule* vala_ccode_method_call_module_new (ValaCCodeGenerator* codegen, ValaCCodeModule* next);
ValaCCodeGenerator* vala_ccode_module_get_codegen (ValaCCodeModule* self);
void vala_ccode_base_module_generate_method_declaration (ValaCCodeBaseModule* self, ValaMethod* m, ValaCCodeDeclarationSpace* decl_space);
static void vala_ccode_method_call_module_add_generic_type_arguments (ValaCCodeMethodCallModule* self, ValaCCodeFunctionCall* ccall, GeeList* type_args, ValaCodeNode* expr);
gint vala_ccode_base_module_get_param_pos (ValaCCodeBaseModule* self, double param_pos, gboolean ellipsis);
ValaLocalVariable* vala_ccode_base_module_get_temp_variable (ValaCCodeBaseModule* self, ValaDataType* type, gboolean value_owned, ValaCodeNode* node_reference);
ValaCCodeExpression* vala_ccode_base_module_get_variable_cexpression (ValaCCodeBaseModule* self, const char* name);
ValaCCodeModule* vala_ccode_module_get_head (ValaCCodeModule* self);
void vala_ccode_module_generate_dynamic_method_wrapper (ValaCCodeModule* self, ValaDynamicMethod* method);
ValaCCodeExpression* vala_ccode_module_get_array_length_cexpression (ValaCCodeModule* self, ValaExpression* array_expr, gint dim);
ValaCCodeExpression* vala_ccode_base_module_get_delegate_target_cexpression (ValaCCodeBaseModule* self, ValaExpression* delegate_expr);
ValaCCodeExpression* vala_ccode_base_module_get_dup_func_expression (ValaCCodeBaseModule* self, ValaDataType* type, ValaSourceReference* source_reference);
ValaCCodeExpression* vala_ccode_base_module_get_destroy_func_expression (ValaCCodeBaseModule* self, ValaDataType* type);
ValaCCodeExpression* vala_ccode_base_module_handle_struct_argument (ValaCCodeBaseModule* self, ValaFormalParameter* param, ValaExpression* arg, ValaCCodeExpression* cexpr);
gboolean vala_ccode_base_module_requires_destroy (ValaCCodeBaseModule* self, ValaDataType* type);
ValaCCodeExpression* vala_ccode_base_module_transform_expression (ValaCCodeBaseModule* self, ValaCCodeExpression* source_cexpr, ValaDataType* expression_type, ValaDataType* target_type, ValaExpression* expr);
ValaCCodeExpression* vala_ccode_base_module_get_unref_expression (ValaCCodeBaseModule* self, ValaCCodeExpression* cvar, ValaDataType* type, ValaExpression* expr);
void vala_ccode_declaration_space_add_include (ValaCCodeDeclarationSpace* self, const char* filename, gboolean local);
static void vala_ccode_method_call_module_real_visit_method_call (ValaCCodeModule* base, ValaMethodCall* expr);
gboolean vala_ccode_base_module_requires_copy (ValaCCodeBaseModule* self, ValaDataType* type);
static int _vala_strcmp0 (const char * str1, const char * str2);



ValaCCodeMethodCallModule* vala_ccode_method_call_module_construct (GType object_type, ValaCCodeGenerator* codegen, ValaCCodeModule* next) {
	ValaCCodeMethodCallModule* self;
	g_return_val_if_fail (codegen != NULL, NULL);
	self = (ValaCCodeMethodCallModule*) vala_ccode_assignment_module_construct (object_type, codegen, next);
	return self;
}


ValaCCodeMethodCallModule* vala_ccode_method_call_module_new (ValaCCodeGenerator* codegen, ValaCCodeModule* next) {
	return vala_ccode_method_call_module_construct (VALA_TYPE_CCODE_METHOD_CALL_MODULE, codegen, next);
}


static void vala_ccode_method_call_module_real_visit_method_call (ValaCCodeModule* base, ValaMethodCall* expr) {
	ValaCCodeMethodCallModule * self;
	ValaCCodeFunctionCall* ccall;
	ValaCCodeFunctionCall* async_call;
	ValaMethod* m;
	ValaDelegate* deleg;
	GeeList* params;
	ValaMemberAccess* _tmp1_;
	ValaExpression* _tmp0_;
	ValaMemberAccess* ma;
	ValaDataType* _tmp2_;
	ValaDataType* itype;
	GeeList* _tmp3_;
	gboolean _tmp20_;
	gboolean _tmp21_;
	GeeHashMap* in_arg_map;
	GeeHashMap* out_arg_map;
	GeeHashMap* _tmp30_;
	gboolean _tmp31_;
	gboolean _tmp32_;
	ValaCCodeExpression* _tmp74_;
	ValaCCodeExpression* ccall_expr;
	ValaCCodeExpression* instance;
	gboolean _tmp78_;
	gboolean _tmp79_;
	gboolean ellipsis;
	gint i;
	gint arg_pos;
	GeeIterator* params_it;
	gboolean _tmp227_;
	gboolean _tmp244_;
	gboolean _tmp261_;
	gboolean _tmp269_;
	gboolean _tmp280_;
	gint last_pos;
	gint min_pos;
	gboolean _tmp300_;
	gboolean _tmp301_;
	gboolean _tmp303_;
	gboolean _tmp304_;
	gboolean _tmp305_;
	self = (ValaCCodeMethodCallModule*) base;
	g_return_if_fail (expr != NULL);
	vala_code_node_accept_children ((ValaCodeNode*) expr, (ValaCodeVisitor*) vala_ccode_module_get_codegen ((ValaCCodeModule*) self));
	/* the bare function call*/
	ccall = vala_ccode_function_call_new (VALA_CCODE_EXPRESSION (vala_code_node_get_ccodenode ((ValaCodeNode*) vala_method_call_get_call (expr))));
	async_call = NULL;
	m = NULL;
	deleg = NULL;
	params = NULL;
	_tmp1_ = NULL;
	_tmp0_ = NULL;
	ma = (_tmp1_ = (_tmp0_ = vala_method_call_get_call (expr), VALA_IS_MEMBER_ACCESS (_tmp0_) ? ((ValaMemberAccess*) _tmp0_) : NULL), (_tmp1_ == NULL) ? NULL : vala_code_node_ref (_tmp1_));
	_tmp2_ = NULL;
	itype = (_tmp2_ = vala_expression_get_value_type (vala_method_call_get_call (expr)), (_tmp2_ == NULL) ? NULL : vala_code_node_ref (_tmp2_));
	_tmp3_ = NULL;
	params = (_tmp3_ = vala_data_type_get_parameters (itype), (params == NULL) ? NULL : (params = (gee_collection_object_unref (params), NULL)), _tmp3_);
	if (VALA_IS_METHOD_TYPE (itype)) {
		ValaMethod* _tmp5_;
		ValaMethod* _tmp4_;
		g_assert (ma != NULL);
		_tmp5_ = NULL;
		_tmp4_ = NULL;
		m = (_tmp5_ = (_tmp4_ = vala_method_type_get_method_symbol (VALA_METHOD_TYPE (itype)), (_tmp4_ == NULL) ? NULL : vala_code_node_ref (_tmp4_)), (m == NULL) ? NULL : (m = (vala_code_node_unref (m), NULL)), _tmp5_);
	} else {
		if (VALA_IS_SIGNAL_TYPE (itype)) {
			ValaSignalType* _tmp6_;
			ValaSignalType* sig_type;
			gboolean _tmp7_;
			gboolean _tmp8_;
			_tmp6_ = NULL;
			sig_type = (_tmp6_ = VALA_SIGNAL_TYPE (itype), (_tmp6_ == NULL) ? NULL : vala_code_node_ref (_tmp6_));
			_tmp7_ = FALSE;
			_tmp8_ = FALSE;
			if (ma != NULL) {
				_tmp8_ = VALA_IS_BASE_ACCESS (vala_member_access_get_inner (ma));
			} else {
				_tmp8_ = FALSE;
			}
			if (_tmp8_) {
				_tmp7_ = vala_signal_get_is_virtual (vala_signal_type_get_signal_symbol (sig_type));
			} else {
				_tmp7_ = FALSE;
			}
			if (_tmp7_) {
				ValaMethod* _tmp9_;
				_tmp9_ = NULL;
				m = (_tmp9_ = vala_signal_get_method_handler (vala_signal_type_get_signal_symbol (sig_type)), (m == NULL) ? NULL : (m = (vala_code_node_unref (m), NULL)), _tmp9_);
			} else {
				ValaCCodeFunctionCall* _tmp11_;
				ValaCCodeFunctionCall* _tmp10_;
				_tmp11_ = NULL;
				_tmp10_ = NULL;
				ccall = (_tmp11_ = (_tmp10_ = VALA_CCODE_FUNCTION_CALL (vala_code_node_get_ccodenode ((ValaCodeNode*) vala_method_call_get_call (expr))), (_tmp10_ == NULL) ? NULL : vala_ccode_node_ref (_tmp10_)), (ccall == NULL) ? NULL : (ccall = (vala_ccode_node_unref (ccall), NULL)), _tmp11_);
			}
			(sig_type == NULL) ? NULL : (sig_type = (vala_code_node_unref (sig_type), NULL));
		} else {
			if (VALA_IS_OBJECT_TYPE (itype)) {
				ValaClass* _tmp12_;
				ValaClass* cl;
				ValaMethod* _tmp14_;
				ValaMethod* _tmp13_;
				ValaCCodeFunctionCall* _tmp17_;
				ValaCCodeIdentifier* _tmp16_;
				char* _tmp15_;
				/* constructor*/
				_tmp12_ = NULL;
				cl = (_tmp12_ = VALA_CLASS (vala_object_type_get_type_symbol (VALA_OBJECT_TYPE (itype))), (_tmp12_ == NULL) ? NULL : vala_code_node_ref (_tmp12_));
				_tmp14_ = NULL;
				_tmp13_ = NULL;
				m = (_tmp14_ = (_tmp13_ = vala_class_get_default_construction_method (cl), (_tmp13_ == NULL) ? NULL : vala_code_node_ref (_tmp13_)), (m == NULL) ? NULL : (m = (vala_code_node_unref (m), NULL)), _tmp14_);
				vala_ccode_base_module_generate_method_declaration ((ValaCCodeBaseModule*) self, m, ((ValaCCodeBaseModule*) self)->source_declarations);
				_tmp17_ = NULL;
				_tmp16_ = NULL;
				_tmp15_ = NULL;
				ccall = (_tmp17_ = vala_ccode_function_call_new ((ValaCCodeExpression*) (_tmp16_ = vala_ccode_identifier_new (_tmp15_ = vala_method_get_real_cname (m)))), (ccall == NULL) ? NULL : (ccall = (vala_ccode_node_unref (ccall), NULL)), _tmp17_);
				(_tmp16_ == NULL) ? NULL : (_tmp16_ = (vala_ccode_node_unref (_tmp16_), NULL));
				_tmp15_ = (g_free (_tmp15_), NULL);
				(cl == NULL) ? NULL : (cl = (vala_code_node_unref (cl), NULL));
			} else {
				if (VALA_IS_DELEGATE_TYPE (itype)) {
					ValaDelegate* _tmp19_;
					ValaDelegate* _tmp18_;
					_tmp19_ = NULL;
					_tmp18_ = NULL;
					deleg = (_tmp19_ = (_tmp18_ = vala_delegate_type_get_delegate_symbol (VALA_DELEGATE_TYPE (itype)), (_tmp18_ == NULL) ? NULL : vala_code_node_ref (_tmp18_)), (deleg == NULL) ? NULL : (deleg = (vala_code_node_unref (deleg), NULL)), _tmp19_);
				}
			}
		}
	}
	_tmp20_ = FALSE;
	_tmp21_ = FALSE;
	if (m != NULL) {
		_tmp21_ = vala_method_get_coroutine (m);
	} else {
		_tmp21_ = FALSE;
	}
	if (_tmp21_) {
		gboolean _tmp22_;
		gboolean _tmp23_;
		gboolean _tmp24_;
		_tmp22_ = FALSE;
		_tmp23_ = FALSE;
		_tmp24_ = FALSE;
		if (((ValaCCodeBaseModule*) self)->current_method != NULL) {
			_tmp24_ = vala_method_get_coroutine (((ValaCCodeBaseModule*) self)->current_method);
		} else {
			_tmp24_ = FALSE;
		}
		if (_tmp24_) {
			_tmp23_ = TRUE;
		} else {
			gboolean _tmp25_;
			_tmp25_ = FALSE;
			if (_vala_strcmp0 (vala_member_access_get_member_name (ma), "begin") == 0) {
				_tmp25_ = vala_expression_get_symbol_reference (vala_member_access_get_inner (ma)) == vala_expression_get_symbol_reference ((ValaExpression*) ma);
			} else {
				_tmp25_ = FALSE;
			}
			_tmp23_ = _tmp25_;
		}
		if (_tmp23_) {
			_tmp22_ = TRUE;
		} else {
			gboolean _tmp26_;
			_tmp26_ = FALSE;
			if (_vala_strcmp0 (vala_member_access_get_member_name (ma), "end") == 0) {
				_tmp26_ = vala_expression_get_symbol_reference (vala_member_access_get_inner (ma)) == vala_expression_get_symbol_reference ((ValaExpression*) ma);
			} else {
				_tmp26_ = FALSE;
			}
			_tmp22_ = _tmp26_;
		}
		_tmp20_ = _tmp22_;
	} else {
		_tmp20_ = FALSE;
	}
	if (_tmp20_) {
		char* async_member_name;
		GeeList* _tmp29_;
		async_member_name = NULL;
		if (vala_expression_get_symbol_reference (vala_member_access_get_inner (ma)) == vala_expression_get_symbol_reference ((ValaExpression*) ma)) {
			char* _tmp28_;
			const char* _tmp27_;
			_tmp28_ = NULL;
			_tmp27_ = NULL;
			async_member_name = (_tmp28_ = (_tmp27_ = vala_member_access_get_member_name (ma), (_tmp27_ == NULL) ? NULL : g_strdup (_tmp27_)), async_member_name = (g_free (async_member_name), NULL), _tmp28_);
		}
		_tmp29_ = NULL;
		params = (_tmp29_ = vala_method_type_get_member_parameters (VALA_METHOD_TYPE (itype), async_member_name), (params == NULL) ? NULL : (params = (gee_collection_object_unref (params), NULL)), _tmp29_);
		async_member_name = (g_free (async_member_name), NULL);
	}
	in_arg_map = NULL;
	out_arg_map = NULL;
	_tmp30_ = NULL;
	in_arg_map = (_tmp30_ = gee_hash_map_new (G_TYPE_INT, NULL, NULL, VALA_TYPE_CCODE_EXPRESSION, (GBoxedCopyFunc) vala_ccode_node_ref, vala_ccode_node_unref, g_direct_hash, g_direct_equal, g_direct_equal), (in_arg_map == NULL) ? NULL : (in_arg_map = (gee_collection_object_unref (in_arg_map), NULL)), _tmp30_);
	_tmp31_ = FALSE;
	_tmp32_ = FALSE;
	if (m != NULL) {
		_tmp32_ = vala_method_get_coroutine (m);
	} else {
		_tmp32_ = FALSE;
	}
	if (_tmp32_) {
		gboolean _tmp33_;
		gboolean _tmp34_;
		gboolean _tmp35_;
		_tmp33_ = FALSE;
		_tmp34_ = FALSE;
		_tmp35_ = FALSE;
		if (((ValaCCodeBaseModule*) self)->current_method != NULL) {
			_tmp35_ = vala_method_get_coroutine (((ValaCCodeBaseModule*) self)->current_method);
		} else {
			_tmp35_ = FALSE;
		}
		if (_tmp35_) {
			_tmp34_ = TRUE;
		} else {
			gboolean _tmp36_;
			_tmp36_ = FALSE;
			if (_vala_strcmp0 (vala_member_access_get_member_name (ma), "begin") == 0) {
				_tmp36_ = vala_expression_get_symbol_reference (vala_member_access_get_inner (ma)) == vala_expression_get_symbol_reference ((ValaExpression*) ma);
			} else {
				_tmp36_ = FALSE;
			}
			_tmp34_ = _tmp36_;
		}
		if (_tmp34_) {
			_tmp33_ = TRUE;
		} else {
			gboolean _tmp37_;
			_tmp37_ = FALSE;
			if (_vala_strcmp0 (vala_member_access_get_member_name (ma), "end") == 0) {
				_tmp37_ = vala_expression_get_symbol_reference (vala_member_access_get_inner (ma)) == vala_expression_get_symbol_reference ((ValaExpression*) ma);
			} else {
				_tmp37_ = FALSE;
			}
			_tmp33_ = _tmp37_;
		}
		_tmp31_ = _tmp33_;
	} else {
		_tmp31_ = FALSE;
	}
	if (_tmp31_) {
		ValaCCodeFunctionCall* _tmp41_;
		ValaCCodeIdentifier* _tmp40_;
		char* _tmp39_;
		char* _tmp38_;
		gboolean _tmp42_;
		/* async call*/
		_tmp41_ = NULL;
		_tmp40_ = NULL;
		_tmp39_ = NULL;
		_tmp38_ = NULL;
		async_call = (_tmp41_ = vala_ccode_function_call_new ((ValaCCodeExpression*) (_tmp40_ = vala_ccode_identifier_new (_tmp39_ = g_strconcat (_tmp38_ = vala_method_get_cname (m), "_async", NULL)))), (async_call == NULL) ? NULL : (async_call = (vala_ccode_node_unref (async_call), NULL)), _tmp41_);
		(_tmp40_ == NULL) ? NULL : (_tmp40_ = (vala_ccode_node_unref (_tmp40_), NULL));
		_tmp39_ = (g_free (_tmp39_), NULL);
		_tmp38_ = (g_free (_tmp38_), NULL);
		_tmp42_ = FALSE;
		if (_vala_strcmp0 (vala_member_access_get_member_name (ma), "begin") == 0) {
			_tmp42_ = vala_expression_get_symbol_reference (vala_member_access_get_inner (ma)) == vala_expression_get_symbol_reference ((ValaExpression*) ma);
		} else {
			_tmp42_ = FALSE;
		}
		if (_tmp42_) {
			ValaCCodeFunctionCall* _tmp44_;
			ValaCCodeFunctionCall* _tmp43_;
			GeeHashMap* _tmp45_;
			/* no finish call*/
			_tmp44_ = NULL;
			_tmp43_ = NULL;
			ccall = (_tmp44_ = (_tmp43_ = async_call, (_tmp43_ == NULL) ? NULL : vala_ccode_node_ref (_tmp43_)), (ccall == NULL) ? NULL : (ccall = (vala_ccode_node_unref (ccall), NULL)), _tmp44_);
			_tmp45_ = NULL;
			out_arg_map = (_tmp45_ = gee_hash_map_new (G_TYPE_INT, NULL, NULL, VALA_TYPE_CCODE_EXPRESSION, (GBoxedCopyFunc) vala_ccode_node_ref, vala_ccode_node_unref, g_direct_hash, g_direct_equal, g_direct_equal), (out_arg_map == NULL) ? NULL : (out_arg_map = (gee_collection_object_unref (out_arg_map), NULL)), _tmp45_);
		} else {
			gboolean _tmp46_;
			_tmp46_ = FALSE;
			if (_vala_strcmp0 (vala_member_access_get_member_name (ma), "end") == 0) {
				_tmp46_ = vala_expression_get_symbol_reference (vala_member_access_get_inner (ma)) == vala_expression_get_symbol_reference ((ValaExpression*) ma);
			} else {
				_tmp46_ = FALSE;
			}
			if (_tmp46_) {
				ValaCCodeFunctionCall* _tmp50_;
				ValaCCodeIdentifier* _tmp49_;
				char* _tmp48_;
				char* _tmp47_;
				GeeHashMap* _tmp52_;
				GeeHashMap* _tmp51_;
				_tmp50_ = NULL;
				_tmp49_ = NULL;
				_tmp48_ = NULL;
				_tmp47_ = NULL;
				ccall = (_tmp50_ = vala_ccode_function_call_new ((ValaCCodeExpression*) (_tmp49_ = vala_ccode_identifier_new (_tmp48_ = g_strconcat (_tmp47_ = vala_method_get_cname (m), "_finish", NULL)))), (ccall == NULL) ? NULL : (ccall = (vala_ccode_node_unref (ccall), NULL)), _tmp50_);
				(_tmp49_ == NULL) ? NULL : (_tmp49_ = (vala_ccode_node_unref (_tmp49_), NULL));
				_tmp48_ = (g_free (_tmp48_), NULL);
				_tmp47_ = (g_free (_tmp47_), NULL);
				_tmp52_ = NULL;
				_tmp51_ = NULL;
				out_arg_map = (_tmp52_ = (_tmp51_ = in_arg_map, (_tmp51_ == NULL) ? NULL : gee_collection_object_ref (_tmp51_)), (out_arg_map == NULL) ? NULL : (out_arg_map = (gee_collection_object_unref (out_arg_map), NULL)), _tmp52_);
			} else {
				ValaCCodeFunctionCall* _tmp56_;
				ValaCCodeIdentifier* _tmp55_;
				char* _tmp54_;
				char* _tmp53_;
				GeeHashMap* _tmp57_;
				_tmp56_ = NULL;
				_tmp55_ = NULL;
				_tmp54_ = NULL;
				_tmp53_ = NULL;
				ccall = (_tmp56_ = vala_ccode_function_call_new ((ValaCCodeExpression*) (_tmp55_ = vala_ccode_identifier_new (_tmp54_ = g_strconcat (_tmp53_ = vala_method_get_cname (m), "_finish", NULL)))), (ccall == NULL) ? NULL : (ccall = (vala_ccode_node_unref (ccall), NULL)), _tmp56_);
				(_tmp55_ == NULL) ? NULL : (_tmp55_ = (vala_ccode_node_unref (_tmp55_), NULL));
				_tmp54_ = (g_free (_tmp54_), NULL);
				_tmp53_ = (g_free (_tmp53_), NULL);
				_tmp57_ = NULL;
				out_arg_map = (_tmp57_ = gee_hash_map_new (G_TYPE_INT, NULL, NULL, VALA_TYPE_CCODE_EXPRESSION, (GBoxedCopyFunc) vala_ccode_node_ref, vala_ccode_node_unref, g_direct_hash, g_direct_equal, g_direct_equal), (out_arg_map == NULL) ? NULL : (out_arg_map = (gee_collection_object_unref (out_arg_map), NULL)), _tmp57_);
			}
		}
	} else {
		GeeHashMap* _tmp59_;
		GeeHashMap* _tmp58_;
		_tmp59_ = NULL;
		_tmp58_ = NULL;
		out_arg_map = (_tmp59_ = (_tmp58_ = in_arg_map, (_tmp58_ == NULL) ? NULL : gee_collection_object_ref (_tmp58_)), (out_arg_map == NULL) ? NULL : (out_arg_map = (gee_collection_object_unref (out_arg_map), NULL)), _tmp59_);
	}
	if (VALA_IS_CREATION_METHOD (m)) {
		ValaCCodeIdentifier* _tmp60_;
		_tmp60_ = NULL;
		vala_ccode_function_call_add_argument (ccall, (ValaCCodeExpression*) (_tmp60_ = vala_ccode_identifier_new ("object_type")));
		(_tmp60_ == NULL) ? NULL : (_tmp60_ = (vala_ccode_node_unref (_tmp60_), NULL));
		{
			GeeList* _tmp61_;
			GeeIterator* _tmp62_;
			GeeIterator* _base_type_it;
			_tmp61_ = NULL;
			_tmp62_ = NULL;
			_base_type_it = (_tmp62_ = gee_iterable_iterator ((GeeIterable*) (_tmp61_ = vala_class_get_base_types (((ValaCCodeBaseModule*) self)->current_class))), (_tmp61_ == NULL) ? NULL : (_tmp61_ = (gee_collection_object_unref (_tmp61_), NULL)), _tmp62_);
			while (TRUE) {
				ValaDataType* base_type;
				if (!gee_iterator_next (_base_type_it)) {
					break;
				}
				base_type = (ValaDataType*) gee_iterator_get (_base_type_it);
				if (VALA_IS_CLASS (vala_data_type_get_data_type (base_type))) {
					GeeList* _tmp63_;
					_tmp63_ = NULL;
					vala_ccode_method_call_module_add_generic_type_arguments (self, ccall, _tmp63_ = vala_data_type_get_type_arguments (base_type), (ValaCodeNode*) expr);
					(_tmp63_ == NULL) ? NULL : (_tmp63_ = (gee_collection_object_unref (_tmp63_), NULL));
					(base_type == NULL) ? NULL : (base_type = (vala_code_node_unref (base_type), NULL));
					break;
				}
				(base_type == NULL) ? NULL : (base_type = (vala_code_node_unref (base_type), NULL));
			}
			(_base_type_it == NULL) ? NULL : (_base_type_it = (gee_collection_object_unref (_base_type_it), NULL));
		}
	} else {
		if (m != NULL) {
			gboolean _tmp64_;
			GeeList* _tmp71_;
			gboolean _tmp72_;
			_tmp64_ = FALSE;
			if (vala_method_get_binding (m) != MEMBER_BINDING_INSTANCE) {
				_tmp64_ = VALA_IS_OBJECT_TYPE_SYMBOL (vala_symbol_get_parent_symbol ((ValaSymbol*) m));
			} else {
				_tmp64_ = FALSE;
			}
			if (_tmp64_) {
				ValaObjectTypeSymbol* _tmp65_;
				ValaObjectTypeSymbol* type_symbol;
				gboolean _tmp66_;
				GeeList* _tmp67_;
				gboolean _tmp68_;
				/* support static methods in generic types*/
				_tmp65_ = NULL;
				type_symbol = (_tmp65_ = VALA_OBJECT_TYPE_SYMBOL (vala_symbol_get_parent_symbol ((ValaSymbol*) m)), (_tmp65_ == NULL) ? NULL : vala_code_node_ref (_tmp65_));
				_tmp66_ = FALSE;
				_tmp67_ = NULL;
				if ((_tmp68_ = gee_collection_get_size ((GeeCollection*) (_tmp67_ = vala_object_type_symbol_get_type_parameters (type_symbol))) > 0, (_tmp67_ == NULL) ? NULL : (_tmp67_ = (gee_collection_object_unref (_tmp67_), NULL)), _tmp68_)) {
					_tmp66_ = VALA_IS_MEMBER_ACCESS (vala_member_access_get_inner (ma));
				} else {
					_tmp66_ = FALSE;
				}
				if (_tmp66_) {
					ValaMemberAccess* _tmp69_;
					ValaMemberAccess* type_ma;
					GeeList* _tmp70_;
					_tmp69_ = NULL;
					type_ma = (_tmp69_ = VALA_MEMBER_ACCESS (vala_member_access_get_inner (ma)), (_tmp69_ == NULL) ? NULL : vala_code_node_ref (_tmp69_));
					_tmp70_ = NULL;
					vala_ccode_method_call_module_add_generic_type_arguments (self, ccall, _tmp70_ = vala_member_access_get_type_arguments (type_ma), (ValaCodeNode*) expr);
					(_tmp70_ == NULL) ? NULL : (_tmp70_ = (gee_collection_object_unref (_tmp70_), NULL));
					(type_ma == NULL) ? NULL : (type_ma = (vala_code_node_unref (type_ma), NULL));
				}
				(type_symbol == NULL) ? NULL : (type_symbol = (vala_code_node_unref (type_symbol), NULL));
			}
			_tmp71_ = NULL;
			if ((_tmp72_ = gee_collection_get_size ((GeeCollection*) (_tmp71_ = vala_method_get_type_parameters (m))) > 0, (_tmp71_ == NULL) ? NULL : (_tmp71_ = (gee_collection_object_unref (_tmp71_), NULL)), _tmp72_)) {
				GeeList* _tmp73_;
				_tmp73_ = NULL;
				vala_ccode_method_call_module_add_generic_type_arguments (self, ccall, _tmp73_ = vala_member_access_get_type_arguments (ma), (ValaCodeNode*) expr);
				(_tmp73_ == NULL) ? NULL : (_tmp73_ = (gee_collection_object_unref (_tmp73_), NULL));
			}
		}
	}
	/* the complete call expression, might include casts, comma expressions, and/or assignments*/
	_tmp74_ = NULL;
	ccall_expr = (_tmp74_ = (ValaCCodeExpression*) ccall, (_tmp74_ == NULL) ? NULL : vala_ccode_node_ref (_tmp74_));
	if (VALA_IS_ARRAY_RESIZE_METHOD (m)) {
		ValaArrayType* _tmp75_;
		ValaArrayType* array_type;
		ValaCCodeIdentifier* _tmp77_;
		char* _tmp76_;
		_tmp75_ = NULL;
		array_type = (_tmp75_ = VALA_ARRAY_TYPE (vala_expression_get_value_type (vala_member_access_get_inner (ma))), (_tmp75_ == NULL) ? NULL : vala_code_node_ref (_tmp75_));
		_tmp77_ = NULL;
		_tmp76_ = NULL;
		gee_map_set ((GeeMap*) in_arg_map, GINT_TO_POINTER (vala_ccode_base_module_get_param_pos ((ValaCCodeBaseModule*) self, (double) 0, FALSE)), (ValaCCodeExpression*) (_tmp77_ = vala_ccode_identifier_new (_tmp76_ = vala_data_type_get_cname (vala_array_type_get_element_type (array_type)))));
		(_tmp77_ == NULL) ? NULL : (_tmp77_ = (vala_ccode_node_unref (_tmp77_), NULL));
		_tmp76_ = (g_free (_tmp76_), NULL);
		(array_type == NULL) ? NULL : (array_type = (vala_code_node_unref (array_type), NULL));
	} else {
		if (VALA_IS_ARRAY_MOVE_METHOD (m)) {
			((ValaCCodeBaseModule*) self)->requires_array_move = TRUE;
		}
	}
	instance = NULL;
	_tmp78_ = FALSE;
	_tmp79_ = FALSE;
	if (m != NULL) {
		_tmp79_ = vala_method_get_binding (m) == MEMBER_BINDING_INSTANCE;
	} else {
		_tmp79_ = FALSE;
	}
	if (_tmp79_) {
		_tmp78_ = !VALA_IS_CREATION_METHOD (m);
	} else {
		_tmp78_ = FALSE;
	}
	if (_tmp78_) {
		ValaCCodeExpression* _tmp81_;
		ValaCCodeExpression* _tmp80_;
		gboolean _tmp82_;
		gboolean _tmp83_;
		ValaStruct* _tmp88_;
		ValaSymbol* _tmp87_;
		ValaStruct* st;
		gboolean _tmp89_;
		_tmp81_ = NULL;
		_tmp80_ = NULL;
		instance = (_tmp81_ = (_tmp80_ = VALA_CCODE_EXPRESSION (vala_code_node_get_ccodenode ((ValaCodeNode*) vala_member_access_get_inner (ma))), (_tmp80_ == NULL) ? NULL : vala_ccode_node_ref (_tmp80_)), (instance == NULL) ? NULL : (instance = (vala_ccode_node_unref (instance), NULL)), _tmp81_);
		_tmp82_ = FALSE;
		_tmp83_ = FALSE;
		if (_vala_strcmp0 (vala_member_access_get_member_name (ma), "begin") == 0) {
			_tmp83_ = TRUE;
		} else {
			_tmp83_ = _vala_strcmp0 (vala_member_access_get_member_name (ma), "end") == 0;
		}
		if (_tmp83_) {
			_tmp82_ = vala_expression_get_symbol_reference (vala_member_access_get_inner (ma)) == vala_expression_get_symbol_reference ((ValaExpression*) ma);
		} else {
			_tmp82_ = FALSE;
		}
		if (_tmp82_) {
			ValaMemberAccess* _tmp84_;
			ValaMemberAccess* inner_ma;
			ValaCCodeExpression* _tmp86_;
			ValaCCodeExpression* _tmp85_;
			_tmp84_ = NULL;
			inner_ma = (_tmp84_ = VALA_MEMBER_ACCESS (vala_member_access_get_inner (ma)), (_tmp84_ == NULL) ? NULL : vala_code_node_ref (_tmp84_));
			_tmp86_ = NULL;
			_tmp85_ = NULL;
			instance = (_tmp86_ = (_tmp85_ = VALA_CCODE_EXPRESSION (vala_code_node_get_ccodenode ((ValaCodeNode*) vala_member_access_get_inner (inner_ma))), (_tmp85_ == NULL) ? NULL : vala_ccode_node_ref (_tmp85_)), (instance == NULL) ? NULL : (instance = (vala_ccode_node_unref (instance), NULL)), _tmp86_);
			(inner_ma == NULL) ? NULL : (inner_ma = (vala_code_node_unref (inner_ma), NULL));
		}
		_tmp88_ = NULL;
		_tmp87_ = NULL;
		st = (_tmp88_ = (_tmp87_ = vala_symbol_get_parent_symbol ((ValaSymbol*) m), VALA_IS_STRUCT (_tmp87_) ? ((ValaStruct*) _tmp87_) : NULL), (_tmp88_ == NULL) ? NULL : vala_code_node_ref (_tmp88_));
		_tmp89_ = FALSE;
		if (st != NULL) {
			_tmp89_ = !vala_struct_is_simple_type (st);
		} else {
			_tmp89_ = FALSE;
		}
		if (_tmp89_) {
			ValaCCodeUnaryExpression* _tmp91_;
			ValaCCodeExpression* _tmp90_;
			ValaCCodeUnaryExpression* unary;
			gboolean _tmp92_;
			/* we need to pass struct instance by reference*/
			_tmp91_ = NULL;
			_tmp90_ = NULL;
			unary = (_tmp91_ = (_tmp90_ = instance, VALA_IS_CCODE_UNARY_EXPRESSION (_tmp90_) ? ((ValaCCodeUnaryExpression*) _tmp90_) : NULL), (_tmp91_ == NULL) ? NULL : vala_ccode_node_ref (_tmp91_));
			_tmp92_ = FALSE;
			if (unary != NULL) {
				_tmp92_ = vala_ccode_unary_expression_get_operator (unary) == VALA_CCODE_UNARY_OPERATOR_POINTER_INDIRECTION;
			} else {
				_tmp92_ = FALSE;
			}
			if (_tmp92_) {
				ValaCCodeExpression* _tmp94_;
				ValaCCodeExpression* _tmp93_;
				/* *expr => expr*/
				_tmp94_ = NULL;
				_tmp93_ = NULL;
				instance = (_tmp94_ = (_tmp93_ = vala_ccode_unary_expression_get_inner (unary), (_tmp93_ == NULL) ? NULL : vala_ccode_node_ref (_tmp93_)), (instance == NULL) ? NULL : (instance = (vala_ccode_node_unref (instance), NULL)), _tmp94_);
			} else {
				gboolean _tmp95_;
				_tmp95_ = FALSE;
				if (VALA_IS_CCODE_IDENTIFIER (instance)) {
					_tmp95_ = TRUE;
				} else {
					_tmp95_ = VALA_IS_CCODE_MEMBER_ACCESS (instance);
				}
				if (_tmp95_) {
					ValaCCodeExpression* _tmp96_;
					_tmp96_ = NULL;
					instance = (_tmp96_ = (ValaCCodeExpression*) vala_ccode_unary_expression_new (VALA_CCODE_UNARY_OPERATOR_ADDRESS_OF, instance), (instance == NULL) ? NULL : (instance = (vala_ccode_node_unref (instance), NULL)), _tmp96_);
				} else {
					ValaCCodeCommaExpression* ccomma;
					ValaLocalVariable* temp_var;
					ValaCCodeAssignment* _tmp98_;
					ValaCCodeExpression* _tmp97_;
					ValaCCodeUnaryExpression* _tmp100_;
					ValaCCodeExpression* _tmp99_;
					ValaCCodeExpression* _tmp102_;
					ValaCCodeExpression* _tmp101_;
					/* if instance is e.g. a function call, we can't take the address of the expression
					 (tmp = expr, &tmp)*/
					ccomma = vala_ccode_comma_expression_new ();
					temp_var = vala_ccode_base_module_get_temp_variable ((ValaCCodeBaseModule*) self, vala_expression_get_target_type (vala_member_access_get_inner (ma)), TRUE, NULL);
					gee_list_insert ((GeeList*) ((ValaCCodeBaseModule*) self)->temp_vars, 0, temp_var);
					_tmp98_ = NULL;
					_tmp97_ = NULL;
					vala_ccode_comma_expression_append_expression (ccomma, (ValaCCodeExpression*) (_tmp98_ = vala_ccode_assignment_new (_tmp97_ = vala_ccode_base_module_get_variable_cexpression ((ValaCCodeBaseModule*) self, vala_symbol_get_name ((ValaSymbol*) temp_var)), instance, VALA_CCODE_ASSIGNMENT_OPERATOR_SIMPLE)));
					(_tmp98_ == NULL) ? NULL : (_tmp98_ = (vala_ccode_node_unref (_tmp98_), NULL));
					(_tmp97_ == NULL) ? NULL : (_tmp97_ = (vala_ccode_node_unref (_tmp97_), NULL));
					_tmp100_ = NULL;
					_tmp99_ = NULL;
					vala_ccode_comma_expression_append_expression (ccomma, (ValaCCodeExpression*) (_tmp100_ = vala_ccode_unary_expression_new (VALA_CCODE_UNARY_OPERATOR_ADDRESS_OF, _tmp99_ = vala_ccode_base_module_get_variable_cexpression ((ValaCCodeBaseModule*) self, vala_symbol_get_name ((ValaSymbol*) temp_var)))));
					(_tmp100_ == NULL) ? NULL : (_tmp100_ = (vala_ccode_node_unref (_tmp100_), NULL));
					(_tmp99_ == NULL) ? NULL : (_tmp99_ = (vala_ccode_node_unref (_tmp99_), NULL));
					_tmp102_ = NULL;
					_tmp101_ = NULL;
					instance = (_tmp102_ = (_tmp101_ = (ValaCCodeExpression*) ccomma, (_tmp101_ == NULL) ? NULL : vala_ccode_node_ref (_tmp101_)), (instance == NULL) ? NULL : (instance = (vala_ccode_node_unref (instance), NULL)), _tmp102_);
					(ccomma == NULL) ? NULL : (ccomma = (vala_ccode_node_unref (ccomma), NULL));
					(temp_var == NULL) ? NULL : (temp_var = (vala_code_node_unref (temp_var), NULL));
				}
			}
			(unary == NULL) ? NULL : (unary = (vala_ccode_node_unref (unary), NULL));
		}
		gee_map_set ((GeeMap*) in_arg_map, GINT_TO_POINTER (vala_ccode_base_module_get_param_pos ((ValaCCodeBaseModule*) self, vala_method_get_cinstance_parameter_position (m), FALSE)), instance);
		gee_map_set ((GeeMap*) out_arg_map, GINT_TO_POINTER (vala_ccode_base_module_get_param_pos ((ValaCCodeBaseModule*) self, vala_method_get_cinstance_parameter_position (m), FALSE)), instance);
		(st == NULL) ? NULL : (st = (vala_code_node_unref (st), NULL));
	} else {
		gboolean _tmp103_;
		_tmp103_ = FALSE;
		if (m != NULL) {
			_tmp103_ = vala_method_get_binding (m) == MEMBER_BINDING_CLASS;
		} else {
			_tmp103_ = FALSE;
		}
		if (_tmp103_) {
			ValaClass* _tmp104_;
			ValaClass* cl;
			ValaCCodeIdentifier* _tmp107_;
			char* _tmp106_;
			char* _tmp105_;
			ValaCCodeFunctionCall* _tmp108_;
			ValaCCodeFunctionCall* cast;
			ValaCCodeExpression* klass;
			_tmp104_ = NULL;
			cl = (_tmp104_ = VALA_CLASS (vala_symbol_get_parent_symbol ((ValaSymbol*) m)), (_tmp104_ == NULL) ? NULL : vala_code_node_ref (_tmp104_));
			_tmp107_ = NULL;
			_tmp106_ = NULL;
			_tmp105_ = NULL;
			_tmp108_ = NULL;
			cast = (_tmp108_ = vala_ccode_function_call_new ((ValaCCodeExpression*) (_tmp107_ = vala_ccode_identifier_new (_tmp106_ = g_strconcat (_tmp105_ = vala_typesymbol_get_upper_case_cname ((ValaTypeSymbol*) cl, NULL), "_CLASS", NULL)))), (_tmp107_ == NULL) ? NULL : (_tmp107_ = (vala_ccode_node_unref (_tmp107_), NULL)), _tmp106_ = (g_free (_tmp106_), NULL), _tmp105_ = (g_free (_tmp105_), NULL), _tmp108_);
			klass = NULL;
			if (vala_member_access_get_inner (ma) == NULL) {
				if (((ValaCCodeBaseModule*) self)->in_static_or_class_ctor) {
					ValaCCodeExpression* _tmp109_;
					/* Accessing the method from a static or class constructor*/
					_tmp109_ = NULL;
					klass = (_tmp109_ = (ValaCCodeExpression*) vala_ccode_identifier_new ("klass"), (klass == NULL) ? NULL : (klass = (vala_ccode_node_unref (klass), NULL)), _tmp109_);
				} else {
					ValaCCodeIdentifier* _tmp110_;
					ValaCCodeFunctionCall* _tmp111_;
					ValaCCodeFunctionCall* k;
					ValaCCodeIdentifier* _tmp112_;
					ValaCCodeExpression* _tmp114_;
					ValaCCodeExpression* _tmp113_;
					/* Accessing the method from within an instance method*/
					_tmp110_ = NULL;
					_tmp111_ = NULL;
					k = (_tmp111_ = vala_ccode_function_call_new ((ValaCCodeExpression*) (_tmp110_ = vala_ccode_identifier_new ("G_OBJECT_GET_CLASS"))), (_tmp110_ == NULL) ? NULL : (_tmp110_ = (vala_ccode_node_unref (_tmp110_), NULL)), _tmp111_);
					_tmp112_ = NULL;
					vala_ccode_function_call_add_argument (k, (ValaCCodeExpression*) (_tmp112_ = vala_ccode_identifier_new ("self")));
					(_tmp112_ == NULL) ? NULL : (_tmp112_ = (vala_ccode_node_unref (_tmp112_), NULL));
					_tmp114_ = NULL;
					_tmp113_ = NULL;
					klass = (_tmp114_ = (_tmp113_ = (ValaCCodeExpression*) k, (_tmp113_ == NULL) ? NULL : vala_ccode_node_ref (_tmp113_)), (klass == NULL) ? NULL : (klass = (vala_ccode_node_unref (klass), NULL)), _tmp114_);
					(k == NULL) ? NULL : (k = (vala_ccode_node_unref (k), NULL));
				}
			} else {
				ValaCCodeIdentifier* _tmp115_;
				ValaCCodeFunctionCall* _tmp116_;
				ValaCCodeFunctionCall* k;
				ValaCCodeExpression* _tmp118_;
				ValaCCodeExpression* _tmp117_;
				/* Accessing the method of an instance*/
				_tmp115_ = NULL;
				_tmp116_ = NULL;
				k = (_tmp116_ = vala_ccode_function_call_new ((ValaCCodeExpression*) (_tmp115_ = vala_ccode_identifier_new ("G_OBJECT_GET_CLASS"))), (_tmp115_ == NULL) ? NULL : (_tmp115_ = (vala_ccode_node_unref (_tmp115_), NULL)), _tmp116_);
				vala_ccode_function_call_add_argument (k, VALA_CCODE_EXPRESSION (vala_code_node_get_ccodenode ((ValaCodeNode*) vala_member_access_get_inner (ma))));
				_tmp118_ = NULL;
				_tmp117_ = NULL;
				klass = (_tmp118_ = (_tmp117_ = (ValaCCodeExpression*) k, (_tmp117_ == NULL) ? NULL : vala_ccode_node_ref (_tmp117_)), (klass == NULL) ? NULL : (klass = (vala_ccode_node_unref (klass), NULL)), _tmp118_);
				(k == NULL) ? NULL : (k = (vala_ccode_node_unref (k), NULL));
			}
			vala_ccode_function_call_add_argument (cast, klass);
			gee_map_set ((GeeMap*) in_arg_map, GINT_TO_POINTER (vala_ccode_base_module_get_param_pos ((ValaCCodeBaseModule*) self, vala_method_get_cinstance_parameter_position (m), FALSE)), (ValaCCodeExpression*) cast);
			gee_map_set ((GeeMap*) out_arg_map, GINT_TO_POINTER (vala_ccode_base_module_get_param_pos ((ValaCCodeBaseModule*) self, vala_method_get_cinstance_parameter_position (m), FALSE)), (ValaCCodeExpression*) cast);
			(cl == NULL) ? NULL : (cl = (vala_code_node_unref (cl), NULL));
			(cast == NULL) ? NULL : (cast = (vala_ccode_node_unref (cast), NULL));
			(klass == NULL) ? NULL : (klass = (vala_ccode_node_unref (klass), NULL));
		}
	}
	if (VALA_IS_ARRAY_MOVE_METHOD (m)) {
		ValaArrayType* _tmp119_;
		ValaArrayType* array_type;
		ValaCCodeIdentifier* _tmp120_;
		ValaCCodeFunctionCall* _tmp121_;
		ValaCCodeFunctionCall* csizeof;
		ValaCCodeIdentifier* _tmp123_;
		char* _tmp122_;
		_tmp119_ = NULL;
		array_type = (_tmp119_ = VALA_ARRAY_TYPE (vala_expression_get_value_type (vala_member_access_get_inner (ma))), (_tmp119_ == NULL) ? NULL : vala_code_node_ref (_tmp119_));
		_tmp120_ = NULL;
		_tmp121_ = NULL;
		csizeof = (_tmp121_ = vala_ccode_function_call_new ((ValaCCodeExpression*) (_tmp120_ = vala_ccode_identifier_new ("sizeof"))), (_tmp120_ == NULL) ? NULL : (_tmp120_ = (vala_ccode_node_unref (_tmp120_), NULL)), _tmp121_);
		_tmp123_ = NULL;
		_tmp122_ = NULL;
		vala_ccode_function_call_add_argument (csizeof, (ValaCCodeExpression*) (_tmp123_ = vala_ccode_identifier_new (_tmp122_ = vala_data_type_get_cname (vala_array_type_get_element_type (array_type)))));
		(_tmp123_ == NULL) ? NULL : (_tmp123_ = (vala_ccode_node_unref (_tmp123_), NULL));
		_tmp122_ = (g_free (_tmp122_), NULL);
		gee_map_set ((GeeMap*) in_arg_map, GINT_TO_POINTER (vala_ccode_base_module_get_param_pos ((ValaCCodeBaseModule*) self, 0.1, FALSE)), (ValaCCodeExpression*) csizeof);
		(array_type == NULL) ? NULL : (array_type = (vala_code_node_unref (array_type), NULL));
		(csizeof == NULL) ? NULL : (csizeof = (vala_ccode_node_unref (csizeof), NULL));
	} else {
		if (VALA_IS_DYNAMIC_METHOD (m)) {
			gint param_nr;
			vala_method_clear_parameters (m);
			param_nr = 1;
			{
				GeeList* _tmp124_;
				GeeIterator* _tmp125_;
				GeeIterator* _arg_it;
				_tmp124_ = NULL;
				_tmp125_ = NULL;
				_arg_it = (_tmp125_ = gee_iterable_iterator ((GeeIterable*) (_tmp124_ = vala_method_call_get_argument_list (expr))), (_tmp124_ == NULL) ? NULL : (_tmp124_ = (gee_collection_object_unref (_tmp124_), NULL)), _tmp125_);
				while (TRUE) {
					ValaExpression* arg;
					ValaUnaryExpression* _tmp127_;
					ValaExpression* _tmp126_;
					ValaUnaryExpression* unary;
					gboolean _tmp128_;
					if (!gee_iterator_next (_arg_it)) {
						break;
					}
					arg = (ValaExpression*) gee_iterator_get (_arg_it);
					_tmp127_ = NULL;
					_tmp126_ = NULL;
					unary = (_tmp127_ = (_tmp126_ = arg, VALA_IS_UNARY_EXPRESSION (_tmp126_) ? ((ValaUnaryExpression*) _tmp126_) : NULL), (_tmp127_ == NULL) ? NULL : vala_code_node_ref (_tmp127_));
					_tmp128_ = FALSE;
					if (unary != NULL) {
						_tmp128_ = vala_unary_expression_get_operator (unary) == VALA_UNARY_OPERATOR_OUT;
					} else {
						_tmp128_ = FALSE;
					}
					if (_tmp128_) {
						char* _tmp129_;
						ValaFormalParameter* _tmp130_;
						ValaFormalParameter* param;
						/* out argument*/
						_tmp129_ = NULL;
						_tmp130_ = NULL;
						param = (_tmp130_ = vala_formal_parameter_new (_tmp129_ = g_strdup_printf ("param%d", param_nr), vala_expression_get_value_type (vala_unary_expression_get_inner (unary)), NULL), _tmp129_ = (g_free (_tmp129_), NULL), _tmp130_);
						vala_formal_parameter_set_direction (param, VALA_PARAMETER_DIRECTION_OUT);
						vala_method_add_parameter (m, param);
						(param == NULL) ? NULL : (param = (vala_code_node_unref (param), NULL));
					} else {
						gboolean _tmp131_;
						_tmp131_ = FALSE;
						if (unary != NULL) {
							_tmp131_ = vala_unary_expression_get_operator (unary) == VALA_UNARY_OPERATOR_REF;
						} else {
							_tmp131_ = FALSE;
						}
						if (_tmp131_) {
							char* _tmp132_;
							ValaFormalParameter* _tmp133_;
							ValaFormalParameter* param;
							/* ref argument*/
							_tmp132_ = NULL;
							_tmp133_ = NULL;
							param = (_tmp133_ = vala_formal_parameter_new (_tmp132_ = g_strdup_printf ("param%d", param_nr), vala_expression_get_value_type (vala_unary_expression_get_inner (unary)), NULL), _tmp132_ = (g_free (_tmp132_), NULL), _tmp133_);
							vala_formal_parameter_set_direction (param, VALA_PARAMETER_DIRECTION_REF);
							vala_method_add_parameter (m, param);
							(param == NULL) ? NULL : (param = (vala_code_node_unref (param), NULL));
						} else {
							ValaFormalParameter* _tmp135_;
							char* _tmp134_;
							/* in argument*/
							_tmp135_ = NULL;
							_tmp134_ = NULL;
							vala_method_add_parameter (m, _tmp135_ = vala_formal_parameter_new (_tmp134_ = g_strdup_printf ("param%d", param_nr), vala_expression_get_value_type (arg), NULL));
							(_tmp135_ == NULL) ? NULL : (_tmp135_ = (vala_code_node_unref (_tmp135_), NULL));
							_tmp134_ = (g_free (_tmp134_), NULL);
						}
					}
					param_nr++;
					(arg == NULL) ? NULL : (arg = (vala_code_node_unref (arg), NULL));
					(unary == NULL) ? NULL : (unary = (vala_code_node_unref (unary), NULL));
				}
				(_arg_it == NULL) ? NULL : (_arg_it = (gee_collection_object_unref (_arg_it), NULL));
			}
			{
				GeeList* _tmp136_;
				GeeIterator* _tmp137_;
				GeeIterator* _param_it;
				_tmp136_ = NULL;
				_tmp137_ = NULL;
				_param_it = (_tmp137_ = gee_iterable_iterator ((GeeIterable*) (_tmp136_ = vala_method_get_parameters (m))), (_tmp136_ == NULL) ? NULL : (_tmp136_ = (gee_collection_object_unref (_tmp136_), NULL)), _tmp137_);
				while (TRUE) {
					ValaFormalParameter* param;
					if (!gee_iterator_next (_param_it)) {
						break;
					}
					param = (ValaFormalParameter*) gee_iterator_get (_param_it);
					vala_code_node_accept ((ValaCodeNode*) param, (ValaCodeVisitor*) vala_ccode_module_get_codegen ((ValaCCodeModule*) self));
					(param == NULL) ? NULL : (param = (vala_code_node_unref (param), NULL));
				}
				(_param_it == NULL) ? NULL : (_param_it = (gee_collection_object_unref (_param_it), NULL));
			}
			vala_ccode_module_generate_dynamic_method_wrapper (vala_ccode_module_get_head ((ValaCCodeModule*) self), VALA_DYNAMIC_METHOD (m));
		} else {
			if (VALA_IS_CREATION_METHOD (m)) {
				ValaCCodeExpression* _tmp142_;
				ValaCCodeCastExpression* _tmp141_;
				char* _tmp140_;
				char* _tmp139_;
				ValaCCodeIdentifier* _tmp138_;
				_tmp142_ = NULL;
				_tmp141_ = NULL;
				_tmp140_ = NULL;
				_tmp139_ = NULL;
				_tmp138_ = NULL;
				ccall_expr = (_tmp142_ = (ValaCCodeExpression*) vala_ccode_assignment_new ((ValaCCodeExpression*) (_tmp138_ = vala_ccode_identifier_new ("self")), (ValaCCodeExpression*) (_tmp141_ = vala_ccode_cast_expression_new ((ValaCCodeExpression*) ccall, _tmp140_ = g_strconcat (_tmp139_ = vala_typesymbol_get_cname ((ValaTypeSymbol*) ((ValaCCodeBaseModule*) self)->current_class, FALSE), "*", NULL))), VALA_CCODE_ASSIGNMENT_OPERATOR_SIMPLE), (ccall_expr == NULL) ? NULL : (ccall_expr = (vala_ccode_node_unref (ccall_expr), NULL)), _tmp142_);
				(_tmp141_ == NULL) ? NULL : (_tmp141_ = (vala_ccode_node_unref (_tmp141_), NULL));
				_tmp140_ = (g_free (_tmp140_), NULL);
				_tmp139_ = (g_free (_tmp139_), NULL);
				(_tmp138_ == NULL) ? NULL : (_tmp138_ = (vala_ccode_node_unref (_tmp138_), NULL));
			}
		}
	}
	ellipsis = FALSE;
	i = 1;
	arg_pos = 0;
	params_it = gee_iterable_iterator ((GeeIterable*) params);
	{
		GeeList* _tmp143_;
		GeeIterator* _tmp144_;
		GeeIterator* _arg_it;
		_tmp143_ = NULL;
		_tmp144_ = NULL;
		_arg_it = (_tmp144_ = gee_iterable_iterator ((GeeIterable*) (_tmp143_ = vala_method_call_get_argument_list (expr))), (_tmp143_ == NULL) ? NULL : (_tmp143_ = (gee_collection_object_unref (_tmp143_), NULL)), _tmp144_);
		while (TRUE) {
			ValaExpression* arg;
			ValaCCodeExpression* _tmp145_;
			ValaCCodeExpression* cexpr;
			GeeHashMap* _tmp146_;
			GeeHashMap* carg_map;
			if (!gee_iterator_next (_arg_it)) {
				break;
			}
			arg = (ValaExpression*) gee_iterator_get (_arg_it);
			_tmp145_ = NULL;
			cexpr = (_tmp145_ = VALA_CCODE_EXPRESSION (vala_code_node_get_ccodenode ((ValaCodeNode*) arg)), (_tmp145_ == NULL) ? NULL : vala_ccode_node_ref (_tmp145_));
			_tmp146_ = NULL;
			carg_map = (_tmp146_ = in_arg_map, (_tmp146_ == NULL) ? NULL : gee_collection_object_ref (_tmp146_));
			if (gee_iterator_next (params_it)) {
				ValaFormalParameter* param;
				gboolean _tmp147_;
				param = (ValaFormalParameter*) gee_iterator_get (params_it);
				_tmp147_ = FALSE;
				if (vala_formal_parameter_get_params_array (param)) {
					_tmp147_ = TRUE;
				} else {
					_tmp147_ = vala_formal_parameter_get_ellipsis (param);
				}
				ellipsis = _tmp147_;
				if (!ellipsis) {
					gboolean _tmp148_;
					gboolean _tmp149_;
					gboolean multiple_cargs;
					gboolean _tmp156_;
					ValaCCodeExpression* _tmp189_;
					gboolean _tmp190_;
					gboolean _tmp196_;
					gboolean _tmp197_;
					gboolean _tmp198_;
					_tmp148_ = FALSE;
					_tmp149_ = FALSE;
					if (vala_formal_parameter_get_async_only (param)) {
						gboolean _tmp150_;
						_tmp150_ = FALSE;
						if (((ValaCCodeBaseModule*) self)->current_method == NULL) {
							_tmp150_ = TRUE;
						} else {
							_tmp150_ = !vala_method_get_coroutine (((ValaCCodeBaseModule*) self)->current_method);
						}
						_tmp149_ = _tmp150_;
					} else {
						_tmp149_ = FALSE;
					}
					if (_tmp149_) {
						gboolean _tmp151_;
						_tmp151_ = FALSE;
						if (ma == NULL) {
							_tmp151_ = TRUE;
						} else {
							_tmp151_ = _vala_strcmp0 (vala_member_access_get_member_name (ma), "begin") != 0;
						}
						_tmp148_ = _tmp151_;
					} else {
						_tmp148_ = FALSE;
					}
					if (_tmp148_) {
						ValaCCodeCommaExpression* comma;
						ValaCCodeExpression* _tmp153_;
						ValaCCodeExpression* _tmp152_;
						/* only skip if we are in a sync function (or property handler) 
						 and not manually starting async 
						 [CCode (async_only = true)] and we're making a
						 * synchronous version of the call.  Emit the arg
						 * at the start of a comma expression so that we
						 * get any side effects but otherwise ignore the
						 * value.
						 */
						comma = vala_ccode_comma_expression_new ();
						vala_ccode_comma_expression_append_expression (comma, VALA_CCODE_EXPRESSION (vala_code_node_get_ccodenode ((ValaCodeNode*) arg)));
						vala_ccode_comma_expression_append_expression (comma, ccall_expr);
						_tmp153_ = NULL;
						_tmp152_ = NULL;
						ccall_expr = (_tmp153_ = (_tmp152_ = (ValaCCodeExpression*) comma, (_tmp152_ == NULL) ? NULL : vala_ccode_node_ref (_tmp152_)), (ccall_expr == NULL) ? NULL : (ccall_expr = (vala_ccode_node_unref (ccall_expr), NULL)), _tmp153_);
						(comma == NULL) ? NULL : (comma = (vala_ccode_node_unref (comma), NULL));
						(param == NULL) ? NULL : (param = (vala_code_node_unref (param), NULL));
						(arg == NULL) ? NULL : (arg = (vala_code_node_unref (arg), NULL));
						(cexpr == NULL) ? NULL : (cexpr = (vala_ccode_node_unref (cexpr), NULL));
						(carg_map == NULL) ? NULL : (carg_map = (gee_collection_object_unref (carg_map), NULL));
						continue;
					}
					/* if the vala argument expands to multiple C arguments,
					 we have to make sure that the C arguments don't depend
					 on each other as there is no guaranteed argument
					 evaluation order
					 http://bugzilla.gnome.org/show_bug.cgi?id=519597*/
					multiple_cargs = FALSE;
					if (vala_formal_parameter_get_direction (param) == VALA_PARAMETER_DIRECTION_OUT) {
						GeeHashMap* _tmp155_;
						GeeHashMap* _tmp154_;
						_tmp155_ = NULL;
						_tmp154_ = NULL;
						carg_map = (_tmp155_ = (_tmp154_ = out_arg_map, (_tmp154_ == NULL) ? NULL : gee_collection_object_ref (_tmp154_)), (carg_map == NULL) ? NULL : (carg_map = (gee_collection_object_unref (carg_map), NULL)), _tmp155_);
					}
					_tmp156_ = FALSE;
					if (!vala_formal_parameter_get_no_array_length (param)) {
						_tmp156_ = VALA_IS_ARRAY_TYPE (vala_formal_parameter_get_parameter_type (param));
					} else {
						_tmp156_ = FALSE;
					}
					if (_tmp156_) {
						ValaArrayType* _tmp157_;
						ValaArrayType* array_type;
						_tmp157_ = NULL;
						array_type = (_tmp157_ = VALA_ARRAY_TYPE (vala_formal_parameter_get_parameter_type (param)), (_tmp157_ == NULL) ? NULL : vala_code_node_ref (_tmp157_));
						{
							gint dim;
							dim = 1;
							{
								gboolean _tmp158_;
								_tmp158_ = TRUE;
								while (TRUE) {
									ValaCCodeExpression* _tmp159_;
									if (!_tmp158_) {
										dim++;
									}
									_tmp158_ = FALSE;
									if (!(dim <= vala_array_type_get_rank (array_type))) {
										break;
									}
									_tmp159_ = NULL;
									gee_map_set ((GeeMap*) carg_map, GINT_TO_POINTER (vala_ccode_base_module_get_param_pos ((ValaCCodeBaseModule*) self, vala_formal_parameter_get_carray_length_parameter_position (param) + (0.01 * dim), FALSE)), _tmp159_ = vala_ccode_module_get_array_length_cexpression (vala_ccode_module_get_head ((ValaCCodeModule*) self), arg, dim));
									(_tmp159_ == NULL) ? NULL : (_tmp159_ = (vala_ccode_node_unref (_tmp159_), NULL));
								}
							}
						}
						multiple_cargs = TRUE;
						(array_type == NULL) ? NULL : (array_type = (vala_code_node_unref (array_type), NULL));
					} else {
						if (VALA_IS_DELEGATE_TYPE (vala_formal_parameter_get_parameter_type (param))) {
							ValaDelegateType* _tmp160_;
							ValaDelegateType* deleg_type;
							ValaDelegate* _tmp161_;
							ValaDelegate* d;
							ValaCCodeExpression* _tmp187_;
							char* _tmp186_;
							_tmp160_ = NULL;
							deleg_type = (_tmp160_ = VALA_DELEGATE_TYPE (vala_formal_parameter_get_parameter_type (param)), (_tmp160_ == NULL) ? NULL : vala_code_node_ref (_tmp160_));
							_tmp161_ = NULL;
							d = (_tmp161_ = vala_delegate_type_get_delegate_symbol (deleg_type), (_tmp161_ == NULL) ? NULL : vala_code_node_ref (_tmp161_));
							if (vala_delegate_get_has_target (d)) {
								ValaCCodeExpression* delegate_target;
								delegate_target = vala_ccode_base_module_get_delegate_target_cexpression ((ValaCCodeBaseModule*) self, arg);
								if (vala_data_type_get_value_owned ((ValaDataType*) deleg_type)) {
									ValaCCodeExpression* delegate_target_destroy_notify;
									ValaMethod* _tmp163_;
									ValaSymbol* _tmp162_;
									ValaMethod* delegate_method;
									ValaLambdaExpression* _tmp165_;
									ValaExpression* _tmp164_;
									ValaLambdaExpression* lambda;
									ValaMemberAccess* _tmp167_;
									ValaExpression* _tmp166_;
									ValaMemberAccess* arg_ma;
									gboolean _tmp168_;
									gboolean _tmp169_;
									delegate_target_destroy_notify = NULL;
									_tmp163_ = NULL;
									_tmp162_ = NULL;
									delegate_method = (_tmp163_ = (_tmp162_ = vala_expression_get_symbol_reference (arg), VALA_IS_METHOD (_tmp162_) ? ((ValaMethod*) _tmp162_) : NULL), (_tmp163_ == NULL) ? NULL : vala_code_node_ref (_tmp163_));
									_tmp165_ = NULL;
									_tmp164_ = NULL;
									lambda = (_tmp165_ = (_tmp164_ = arg, VALA_IS_LAMBDA_EXPRESSION (_tmp164_) ? ((ValaLambdaExpression*) _tmp164_) : NULL), (_tmp165_ == NULL) ? NULL : vala_code_node_ref (_tmp165_));
									_tmp167_ = NULL;
									_tmp166_ = NULL;
									arg_ma = (_tmp167_ = (_tmp166_ = arg, VALA_IS_MEMBER_ACCESS (_tmp166_) ? ((ValaMemberAccess*) _tmp166_) : NULL), (_tmp167_ == NULL) ? NULL : vala_code_node_ref (_tmp167_));
									_tmp168_ = FALSE;
									_tmp169_ = FALSE;
									if (lambda != NULL) {
										_tmp169_ = ((ValaCCodeBaseModule*) self)->current_method != NULL;
									} else {
										_tmp169_ = FALSE;
									}
									if (_tmp169_) {
										_tmp168_ = vala_method_get_binding (((ValaCCodeBaseModule*) self)->current_method) == MEMBER_BINDING_INSTANCE;
									} else {
										_tmp168_ = FALSE;
									}
									if (_tmp168_) {
										ValaCCodeExpression* _tmp170_;
										ValaCCodeFunctionCall* _tmp171_;
										ValaCCodeFunctionCall* ref_call;
										ValaCCodeExpression* _tmp173_;
										ValaCCodeExpression* _tmp172_;
										ValaCCodeExpression* _tmp174_;
										/* type of delegate target is same as `this'
										 for lambda expressions in instance methods*/
										_tmp170_ = NULL;
										_tmp171_ = NULL;
										ref_call = (_tmp171_ = vala_ccode_function_call_new (_tmp170_ = vala_ccode_base_module_get_dup_func_expression ((ValaCCodeBaseModule*) self, vala_formal_parameter_get_parameter_type (vala_method_get_this_parameter (((ValaCCodeBaseModule*) self)->current_method)), vala_code_node_get_source_reference ((ValaCodeNode*) arg))), (_tmp170_ == NULL) ? NULL : (_tmp170_ = (vala_ccode_node_unref (_tmp170_), NULL)), _tmp171_);
										vala_ccode_function_call_add_argument (ref_call, delegate_target);
										_tmp173_ = NULL;
										_tmp172_ = NULL;
										delegate_target = (_tmp173_ = (_tmp172_ = (ValaCCodeExpression*) ref_call, (_tmp172_ == NULL) ? NULL : vala_ccode_node_ref (_tmp172_)), (delegate_target == NULL) ? NULL : (delegate_target = (vala_ccode_node_unref (delegate_target), NULL)), _tmp173_);
										_tmp174_ = NULL;
										delegate_target_destroy_notify = (_tmp174_ = vala_ccode_base_module_get_destroy_func_expression ((ValaCCodeBaseModule*) self, vala_formal_parameter_get_parameter_type (vala_method_get_this_parameter (((ValaCCodeBaseModule*) self)->current_method))), (delegate_target_destroy_notify == NULL) ? NULL : (delegate_target_destroy_notify = (vala_ccode_node_unref (delegate_target_destroy_notify), NULL)), _tmp174_);
										(ref_call == NULL) ? NULL : (ref_call = (vala_ccode_node_unref (ref_call), NULL));
									} else {
										gboolean _tmp175_;
										gboolean _tmp176_;
										gboolean _tmp177_;
										gboolean _tmp178_;
										gboolean _tmp179_;
										_tmp175_ = FALSE;
										_tmp176_ = FALSE;
										_tmp177_ = FALSE;
										_tmp178_ = FALSE;
										_tmp179_ = FALSE;
										if (delegate_method != NULL) {
											_tmp179_ = vala_method_get_binding (delegate_method) == MEMBER_BINDING_INSTANCE;
										} else {
											_tmp179_ = FALSE;
										}
										if (_tmp179_) {
											_tmp178_ = arg_ma != NULL;
										} else {
											_tmp178_ = FALSE;
										}
										if (_tmp178_) {
											_tmp177_ = vala_member_access_get_inner (arg_ma) != NULL;
										} else {
											_tmp177_ = FALSE;
										}
										if (_tmp177_) {
											_tmp176_ = vala_data_type_get_data_type (vala_expression_get_value_type (vala_member_access_get_inner (arg_ma))) != NULL;
										} else {
											_tmp176_ = FALSE;
										}
										if (_tmp176_) {
											_tmp175_ = vala_typesymbol_is_reference_counting (vala_data_type_get_data_type (vala_expression_get_value_type (vala_member_access_get_inner (arg_ma))));
										} else {
											_tmp175_ = FALSE;
										}
										if (_tmp175_) {
											ValaCCodeExpression* _tmp180_;
											ValaCCodeFunctionCall* _tmp181_;
											ValaCCodeFunctionCall* ref_call;
											ValaCCodeExpression* _tmp183_;
											ValaCCodeExpression* _tmp182_;
											ValaCCodeExpression* _tmp184_;
											_tmp180_ = NULL;
											_tmp181_ = NULL;
											ref_call = (_tmp181_ = vala_ccode_function_call_new (_tmp180_ = vala_ccode_base_module_get_dup_func_expression ((ValaCCodeBaseModule*) self, vala_expression_get_value_type (vala_member_access_get_inner (arg_ma)), vala_code_node_get_source_reference ((ValaCodeNode*) arg))), (_tmp180_ == NULL) ? NULL : (_tmp180_ = (vala_ccode_node_unref (_tmp180_), NULL)), _tmp181_);
											vala_ccode_function_call_add_argument (ref_call, delegate_target);
											_tmp183_ = NULL;
											_tmp182_ = NULL;
											delegate_target = (_tmp183_ = (_tmp182_ = (ValaCCodeExpression*) ref_call, (_tmp182_ == NULL) ? NULL : vala_ccode_node_ref (_tmp182_)), (delegate_target == NULL) ? NULL : (delegate_target = (vala_ccode_node_unref (delegate_target), NULL)), _tmp183_);
											_tmp184_ = NULL;
											delegate_target_destroy_notify = (_tmp184_ = vala_ccode_base_module_get_destroy_func_expression ((ValaCCodeBaseModule*) self, vala_expression_get_value_type (vala_member_access_get_inner (arg_ma))), (delegate_target_destroy_notify == NULL) ? NULL : (delegate_target_destroy_notify = (vala_ccode_node_unref (delegate_target_destroy_notify), NULL)), _tmp184_);
											(ref_call == NULL) ? NULL : (ref_call = (vala_ccode_node_unref (ref_call), NULL));
										} else {
											ValaCCodeExpression* _tmp185_;
											_tmp185_ = NULL;
											delegate_target_destroy_notify = (_tmp185_ = (ValaCCodeExpression*) vala_ccode_constant_new ("NULL"), (delegate_target_destroy_notify == NULL) ? NULL : (delegate_target_destroy_notify = (vala_ccode_node_unref (delegate_target_destroy_notify), NULL)), _tmp185_);
										}
									}
									gee_map_set ((GeeMap*) carg_map, GINT_TO_POINTER (vala_ccode_base_module_get_param_pos ((ValaCCodeBaseModule*) self, vala_formal_parameter_get_cdelegate_target_parameter_position (param) + 0.01, FALSE)), delegate_target_destroy_notify);
									(delegate_target_destroy_notify == NULL) ? NULL : (delegate_target_destroy_notify = (vala_ccode_node_unref (delegate_target_destroy_notify), NULL));
									(delegate_method == NULL) ? NULL : (delegate_method = (vala_code_node_unref (delegate_method), NULL));
									(lambda == NULL) ? NULL : (lambda = (vala_code_node_unref (lambda), NULL));
									(arg_ma == NULL) ? NULL : (arg_ma = (vala_code_node_unref (arg_ma), NULL));
								}
								gee_map_set ((GeeMap*) carg_map, GINT_TO_POINTER (vala_ccode_base_module_get_param_pos ((ValaCCodeBaseModule*) self, vala_formal_parameter_get_cdelegate_target_parameter_position (param), FALSE)), delegate_target);
								multiple_cargs = TRUE;
								(delegate_target == NULL) ? NULL : (delegate_target = (vala_ccode_node_unref (delegate_target), NULL));
							}
							_tmp187_ = NULL;
							_tmp186_ = NULL;
							cexpr = (_tmp187_ = (ValaCCodeExpression*) vala_ccode_cast_expression_new (cexpr, _tmp186_ = vala_data_type_get_cname ((ValaDataType*) deleg_type)), (cexpr == NULL) ? NULL : (cexpr = (vala_ccode_node_unref (cexpr), NULL)), _tmp187_);
							_tmp186_ = (g_free (_tmp186_), NULL);
							(deleg_type == NULL) ? NULL : (deleg_type = (vala_code_node_unref (deleg_type), NULL));
							(d == NULL) ? NULL : (d = (vala_code_node_unref (d), NULL));
						} else {
							if (VALA_IS_METHOD_TYPE (vala_formal_parameter_get_parameter_type (param))) {
								ValaCCodeExpression* _tmp188_;
								_tmp188_ = NULL;
								gee_map_set ((GeeMap*) carg_map, GINT_TO_POINTER (vala_ccode_base_module_get_param_pos ((ValaCCodeBaseModule*) self, vala_formal_parameter_get_cdelegate_target_parameter_position (param), FALSE)), _tmp188_ = vala_ccode_base_module_get_delegate_target_cexpression ((ValaCCodeBaseModule*) self, arg));
								(_tmp188_ == NULL) ? NULL : (_tmp188_ = (vala_ccode_node_unref (_tmp188_), NULL));
								multiple_cargs = TRUE;
							}
						}
					}
					_tmp189_ = NULL;
					cexpr = (_tmp189_ = vala_ccode_base_module_handle_struct_argument ((ValaCCodeBaseModule*) self, param, arg, cexpr), (cexpr == NULL) ? NULL : (cexpr = (vala_ccode_node_unref (cexpr), NULL)), _tmp189_);
					_tmp190_ = FALSE;
					if (multiple_cargs) {
						_tmp190_ = VALA_IS_METHOD_CALL (arg);
					} else {
						_tmp190_ = FALSE;
					}
					if (_tmp190_) {
						ValaCCodeCommaExpression* ccomma;
						ValaLocalVariable* temp_decl;
						ValaCCodeAssignment* _tmp192_;
						ValaCCodeExpression* _tmp191_;
						ValaCCodeExpression* _tmp193_;
						ValaCCodeExpression* _tmp195_;
						ValaCCodeExpression* _tmp194_;
						/* if vala argument is invocation expression
						 the auxiliary C argument(s) will depend on the main C argument
						 (tmp = arg1, call (tmp, arg2, arg3,...))*/
						ccomma = vala_ccode_comma_expression_new ();
						temp_decl = vala_ccode_base_module_get_temp_variable ((ValaCCodeBaseModule*) self, vala_expression_get_value_type (arg), TRUE, NULL);
						gee_list_insert ((GeeList*) ((ValaCCodeBaseModule*) self)->temp_vars, 0, temp_decl);
						_tmp192_ = NULL;
						_tmp191_ = NULL;
						vala_ccode_comma_expression_append_expression (ccomma, (ValaCCodeExpression*) (_tmp192_ = vala_ccode_assignment_new (_tmp191_ = vala_ccode_base_module_get_variable_cexpression ((ValaCCodeBaseModule*) self, vala_symbol_get_name ((ValaSymbol*) temp_decl)), cexpr, VALA_CCODE_ASSIGNMENT_OPERATOR_SIMPLE)));
						(_tmp192_ == NULL) ? NULL : (_tmp192_ = (vala_ccode_node_unref (_tmp192_), NULL));
						(_tmp191_ == NULL) ? NULL : (_tmp191_ = (vala_ccode_node_unref (_tmp191_), NULL));
						_tmp193_ = NULL;
						cexpr = (_tmp193_ = (ValaCCodeExpression*) vala_ccode_identifier_new (vala_symbol_get_name ((ValaSymbol*) temp_decl)), (cexpr == NULL) ? NULL : (cexpr = (vala_ccode_node_unref (cexpr), NULL)), _tmp193_);
						vala_ccode_comma_expression_append_expression (ccomma, ccall_expr);
						_tmp195_ = NULL;
						_tmp194_ = NULL;
						ccall_expr = (_tmp195_ = (_tmp194_ = (ValaCCodeExpression*) ccomma, (_tmp194_ == NULL) ? NULL : vala_ccode_node_ref (_tmp194_)), (ccall_expr == NULL) ? NULL : (ccall_expr = (vala_ccode_node_unref (ccall_expr), NULL)), _tmp195_);
						(ccomma == NULL) ? NULL : (ccomma = (vala_ccode_node_unref (ccomma), NULL));
						(temp_decl == NULL) ? NULL : (temp_decl = (vala_code_node_unref (temp_decl), NULL));
					}
					_tmp196_ = FALSE;
					_tmp197_ = FALSE;
					_tmp198_ = FALSE;
					if (vala_formal_parameter_get_direction (param) != VALA_PARAMETER_DIRECTION_IN) {
						_tmp198_ = vala_ccode_base_module_requires_destroy ((ValaCCodeBaseModule*) self, vala_expression_get_value_type (arg));
					} else {
						_tmp198_ = FALSE;
					}
					if (_tmp198_) {
						gboolean _tmp199_;
						_tmp199_ = FALSE;
						if (vala_formal_parameter_get_direction (param) == VALA_PARAMETER_DIRECTION_OUT) {
							_tmp199_ = TRUE;
						} else {
							_tmp199_ = !vala_data_type_get_value_owned (vala_formal_parameter_get_parameter_type (param));
						}
						_tmp197_ = _tmp199_;
					} else {
						_tmp197_ = FALSE;
					}
					if (_tmp197_) {
						_tmp196_ = !VALA_IS_ARRAY_TYPE (vala_formal_parameter_get_parameter_type (param));
					} else {
						_tmp196_ = FALSE;
					}
					/* unref old value for non-null non-weak ref/out arguments
					 disabled for arrays for now as that requires special handling
					 (ret_tmp = call (&tmp), var1 = (assign_tmp = dup (tmp), free (var1), assign_tmp), ret_tmp)*/
					if (_tmp196_) {
						ValaUnaryExpression* _tmp200_;
						ValaUnaryExpression* unary;
						ValaCCodeCommaExpression* ccomma;
						ValaLocalVariable* temp_var;
						ValaCCodeExpression* _tmp202_;
						ValaCCodeExpression* _tmp201_;
						ValaLocalVariable* ret_temp_var;
						ValaDataType* _tmp207_;
						gboolean _tmp208_;
						ValaCCodeCommaExpression* cassign_comma;
						ValaLocalVariable* assign_temp_var;
						ValaCCodeAssignment* _tmp216_;
						ValaCCodeExpression* _tmp215_;
						ValaCCodeExpression* _tmp214_;
						ValaCCodeExpression* _tmp213_;
						ValaCCodeExpression* _tmp217_;
						ValaCCodeExpression* _tmp218_;
						ValaCCodeAssignment* _tmp219_;
						ValaDataType* _tmp220_;
						gboolean _tmp221_;
						ValaCCodeExpression* _tmp224_;
						ValaCCodeExpression* _tmp223_;
						_tmp200_ = NULL;
						unary = (_tmp200_ = VALA_UNARY_EXPRESSION (arg), (_tmp200_ == NULL) ? NULL : vala_code_node_ref (_tmp200_));
						ccomma = vala_ccode_comma_expression_new ();
						temp_var = vala_ccode_base_module_get_temp_variable ((ValaCCodeBaseModule*) self, vala_formal_parameter_get_parameter_type (param), vala_data_type_get_value_owned (vala_formal_parameter_get_parameter_type (param)), NULL);
						gee_list_insert ((GeeList*) ((ValaCCodeBaseModule*) self)->temp_vars, 0, temp_var);
						_tmp202_ = NULL;
						_tmp201_ = NULL;
						cexpr = (_tmp202_ = (ValaCCodeExpression*) vala_ccode_unary_expression_new (VALA_CCODE_UNARY_OPERATOR_ADDRESS_OF, _tmp201_ = vala_ccode_base_module_get_variable_cexpression ((ValaCCodeBaseModule*) self, vala_symbol_get_name ((ValaSymbol*) temp_var))), (cexpr == NULL) ? NULL : (cexpr = (vala_ccode_node_unref (cexpr), NULL)), _tmp202_);
						(_tmp201_ == NULL) ? NULL : (_tmp201_ = (vala_ccode_node_unref (_tmp201_), NULL));
						if (vala_formal_parameter_get_direction (param) == VALA_PARAMETER_DIRECTION_REF) {
							ValaCCodeCommaExpression* crefcomma;
							ValaCCodeAssignment* _tmp204_;
							ValaCCodeExpression* _tmp203_;
							ValaCCodeExpression* _tmp206_;
							ValaCCodeExpression* _tmp205_;
							crefcomma = vala_ccode_comma_expression_new ();
							_tmp204_ = NULL;
							_tmp203_ = NULL;
							vala_ccode_comma_expression_append_expression (crefcomma, (ValaCCodeExpression*) (_tmp204_ = vala_ccode_assignment_new (_tmp203_ = vala_ccode_base_module_get_variable_cexpression ((ValaCCodeBaseModule*) self, vala_symbol_get_name ((ValaSymbol*) temp_var)), VALA_CCODE_EXPRESSION (vala_code_node_get_ccodenode ((ValaCodeNode*) vala_unary_expression_get_inner (unary))), VALA_CCODE_ASSIGNMENT_OPERATOR_SIMPLE)));
							(_tmp204_ == NULL) ? NULL : (_tmp204_ = (vala_ccode_node_unref (_tmp204_), NULL));
							(_tmp203_ == NULL) ? NULL : (_tmp203_ = (vala_ccode_node_unref (_tmp203_), NULL));
							vala_ccode_comma_expression_append_expression (crefcomma, cexpr);
							_tmp206_ = NULL;
							_tmp205_ = NULL;
							cexpr = (_tmp206_ = (_tmp205_ = (ValaCCodeExpression*) crefcomma, (_tmp205_ == NULL) ? NULL : vala_ccode_node_ref (_tmp205_)), (cexpr == NULL) ? NULL : (cexpr = (vala_ccode_node_unref (cexpr), NULL)), _tmp206_);
							(crefcomma == NULL) ? NULL : (crefcomma = (vala_ccode_node_unref (crefcomma), NULL));
						}
						/* call function*/
						ret_temp_var = NULL;
						_tmp207_ = NULL;
						if ((_tmp208_ = VALA_IS_VOID_TYPE (_tmp207_ = vala_data_type_get_return_type (itype)), (_tmp207_ == NULL) ? NULL : (_tmp207_ = (vala_code_node_unref (_tmp207_), NULL)), _tmp208_)) {
							vala_ccode_comma_expression_append_expression (ccomma, ccall_expr);
						} else {
							ValaLocalVariable* _tmp210_;
							ValaDataType* _tmp209_;
							ValaCCodeAssignment* _tmp212_;
							ValaCCodeExpression* _tmp211_;
							_tmp210_ = NULL;
							_tmp209_ = NULL;
							ret_temp_var = (_tmp210_ = vala_ccode_base_module_get_temp_variable ((ValaCCodeBaseModule*) self, _tmp209_ = vala_data_type_get_return_type (itype), TRUE, NULL), (ret_temp_var == NULL) ? NULL : (ret_temp_var = (vala_code_node_unref (ret_temp_var), NULL)), _tmp210_);
							(_tmp209_ == NULL) ? NULL : (_tmp209_ = (vala_code_node_unref (_tmp209_), NULL));
							gee_list_insert ((GeeList*) ((ValaCCodeBaseModule*) self)->temp_vars, 0, ret_temp_var);
							_tmp212_ = NULL;
							_tmp211_ = NULL;
							vala_ccode_comma_expression_append_expression (ccomma, (ValaCCodeExpression*) (_tmp212_ = vala_ccode_assignment_new (_tmp211_ = vala_ccode_base_module_get_variable_cexpression ((ValaCCodeBaseModule*) self, vala_symbol_get_name ((ValaSymbol*) ret_temp_var)), ccall_expr, VALA_CCODE_ASSIGNMENT_OPERATOR_SIMPLE)));
							(_tmp212_ == NULL) ? NULL : (_tmp212_ = (vala_ccode_node_unref (_tmp212_), NULL));
							(_tmp211_ == NULL) ? NULL : (_tmp211_ = (vala_ccode_node_unref (_tmp211_), NULL));
						}
						cassign_comma = vala_ccode_comma_expression_new ();
						assign_temp_var = vala_ccode_base_module_get_temp_variable ((ValaCCodeBaseModule*) self, vala_expression_get_value_type (vala_unary_expression_get_inner (unary)), vala_data_type_get_value_owned (vala_expression_get_value_type (vala_unary_expression_get_inner (unary))), NULL);
						gee_list_insert ((GeeList*) ((ValaCCodeBaseModule*) self)->temp_vars, 0, assign_temp_var);
						_tmp216_ = NULL;
						_tmp215_ = NULL;
						_tmp214_ = NULL;
						_tmp213_ = NULL;
						vala_ccode_comma_expression_append_expression (cassign_comma, (ValaCCodeExpression*) (_tmp216_ = vala_ccode_assignment_new (_tmp213_ = vala_ccode_base_module_get_variable_cexpression ((ValaCCodeBaseModule*) self, vala_symbol_get_name ((ValaSymbol*) assign_temp_var)), _tmp215_ = vala_ccode_base_module_transform_expression ((ValaCCodeBaseModule*) self, _tmp214_ = vala_ccode_base_module_get_variable_cexpression ((ValaCCodeBaseModule*) self, vala_symbol_get_name ((ValaSymbol*) temp_var)), vala_formal_parameter_get_parameter_type (param), vala_expression_get_value_type (vala_unary_expression_get_inner (unary)), arg), VALA_CCODE_ASSIGNMENT_OPERATOR_SIMPLE)));
						(_tmp216_ == NULL) ? NULL : (_tmp216_ = (vala_ccode_node_unref (_tmp216_), NULL));
						(_tmp215_ == NULL) ? NULL : (_tmp215_ = (vala_ccode_node_unref (_tmp215_), NULL));
						(_tmp214_ == NULL) ? NULL : (_tmp214_ = (vala_ccode_node_unref (_tmp214_), NULL));
						(_tmp213_ == NULL) ? NULL : (_tmp213_ = (vala_ccode_node_unref (_tmp213_), NULL));
						/* unref old value*/
						_tmp217_ = NULL;
						vala_ccode_comma_expression_append_expression (cassign_comma, _tmp217_ = vala_ccode_base_module_get_unref_expression ((ValaCCodeBaseModule*) self, VALA_CCODE_EXPRESSION (vala_code_node_get_ccodenode ((ValaCodeNode*) vala_unary_expression_get_inner (unary))), vala_expression_get_value_type (arg), arg));
						(_tmp217_ == NULL) ? NULL : (_tmp217_ = (vala_ccode_node_unref (_tmp217_), NULL));
						_tmp218_ = NULL;
						vala_ccode_comma_expression_append_expression (cassign_comma, _tmp218_ = vala_ccode_base_module_get_variable_cexpression ((ValaCCodeBaseModule*) self, vala_symbol_get_name ((ValaSymbol*) assign_temp_var)));
						(_tmp218_ == NULL) ? NULL : (_tmp218_ = (vala_ccode_node_unref (_tmp218_), NULL));
						/* assign new value*/
						_tmp219_ = NULL;
						vala_ccode_comma_expression_append_expression (ccomma, (ValaCCodeExpression*) (_tmp219_ = vala_ccode_assignment_new (VALA_CCODE_EXPRESSION (vala_code_node_get_ccodenode ((ValaCodeNode*) vala_unary_expression_get_inner (unary))), (ValaCCodeExpression*) cassign_comma, VALA_CCODE_ASSIGNMENT_OPERATOR_SIMPLE)));
						(_tmp219_ == NULL) ? NULL : (_tmp219_ = (vala_ccode_node_unref (_tmp219_), NULL));
						/* return value*/
						_tmp220_ = NULL;
						if ((_tmp221_ = !VALA_IS_VOID_TYPE (_tmp220_ = vala_data_type_get_return_type (itype)), (_tmp220_ == NULL) ? NULL : (_tmp220_ = (vala_code_node_unref (_tmp220_), NULL)), _tmp221_)) {
							ValaCCodeExpression* _tmp222_;
							_tmp222_ = NULL;
							vala_ccode_comma_expression_append_expression (ccomma, _tmp222_ = vala_ccode_base_module_get_variable_cexpression ((ValaCCodeBaseModule*) self, vala_symbol_get_name ((ValaSymbol*) ret_temp_var)));
							(_tmp222_ == NULL) ? NULL : (_tmp222_ = (vala_ccode_node_unref (_tmp222_), NULL));
						}
						_tmp224_ = NULL;
						_tmp223_ = NULL;
						ccall_expr = (_tmp224_ = (_tmp223_ = (ValaCCodeExpression*) ccomma, (_tmp223_ == NULL) ? NULL : vala_ccode_node_ref (_tmp223_)), (ccall_expr == NULL) ? NULL : (ccall_expr = (vala_ccode_node_unref (ccall_expr), NULL)), _tmp224_);
						(unary == NULL) ? NULL : (unary = (vala_code_node_unref (unary), NULL));
						(ccomma == NULL) ? NULL : (ccomma = (vala_ccode_node_unref (ccomma), NULL));
						(temp_var == NULL) ? NULL : (temp_var = (vala_code_node_unref (temp_var), NULL));
						(ret_temp_var == NULL) ? NULL : (ret_temp_var = (vala_code_node_unref (ret_temp_var), NULL));
						(cassign_comma == NULL) ? NULL : (cassign_comma = (vala_ccode_node_unref (cassign_comma), NULL));
						(assign_temp_var == NULL) ? NULL : (assign_temp_var = (vala_code_node_unref (assign_temp_var), NULL));
					}
					if (vala_formal_parameter_get_ctype (param) != NULL) {
						ValaCCodeExpression* _tmp225_;
						_tmp225_ = NULL;
						cexpr = (_tmp225_ = (ValaCCodeExpression*) vala_ccode_cast_expression_new (cexpr, vala_formal_parameter_get_ctype (param)), (cexpr == NULL) ? NULL : (cexpr = (vala_ccode_node_unref (cexpr), NULL)), _tmp225_);
					}
				}
				arg_pos = vala_ccode_base_module_get_param_pos ((ValaCCodeBaseModule*) self, vala_formal_parameter_get_cparameter_position (param), ellipsis);
				(param == NULL) ? NULL : (param = (vala_code_node_unref (param), NULL));
			} else {
				/* default argument position*/
				arg_pos = vala_ccode_base_module_get_param_pos ((ValaCCodeBaseModule*) self, (double) i, ellipsis);
			}
			gee_map_set ((GeeMap*) carg_map, GINT_TO_POINTER (arg_pos), cexpr);
			i++;
			(arg == NULL) ? NULL : (arg = (vala_code_node_unref (arg), NULL));
			(cexpr == NULL) ? NULL : (cexpr = (vala_ccode_node_unref (cexpr), NULL));
			(carg_map == NULL) ? NULL : (carg_map = (gee_collection_object_unref (carg_map), NULL));
		}
		(_arg_it == NULL) ? NULL : (_arg_it = (gee_collection_object_unref (_arg_it), NULL));
	}
	if (gee_iterator_next (params_it)) {
		ValaFormalParameter* param;
		gboolean _tmp226_;
		param = (ValaFormalParameter*) gee_iterator_get (params_it);
		_tmp226_ = FALSE;
		if (vala_formal_parameter_get_params_array (param)) {
			_tmp226_ = TRUE;
		} else {
			_tmp226_ = vala_formal_parameter_get_ellipsis (param);
		}
		/* if there are more parameters than arguments,
		 * the additional parameter is an ellipsis parameter
		 * otherwise there is a bug in the semantic analyzer
		 */
		g_assert (_tmp226_);
		ellipsis = TRUE;
		(param == NULL) ? NULL : (param = (vala_code_node_unref (param), NULL));
	}
	_tmp227_ = FALSE;
	if (m != NULL) {
		_tmp227_ = VALA_IS_ARRAY_TYPE (vala_method_get_return_type (m));
	} else {
		_tmp227_ = FALSE;
	}
	/* add length argument for methods returning arrays */
	if (_tmp227_) {
		ValaArrayType* _tmp228_;
		ValaArrayType* array_type;
		_tmp228_ = NULL;
		array_type = (_tmp228_ = VALA_ARRAY_TYPE (vala_method_get_return_type (m)), (_tmp228_ == NULL) ? NULL : vala_code_node_ref (_tmp228_));
		{
			gint dim;
			dim = 1;
			{
				gboolean _tmp229_;
				_tmp229_ = TRUE;
				while (TRUE) {
					if (!_tmp229_) {
						dim++;
					}
					_tmp229_ = FALSE;
					if (!(dim <= vala_array_type_get_rank (array_type))) {
						break;
					}
					if (vala_method_get_array_null_terminated (m)) {
						ValaDataType* _tmp230_;
						ValaLocalVariable* _tmp231_;
						ValaLocalVariable* temp_var;
						ValaCCodeExpression* temp_ref;
						ValaCCodeExpression* _tmp232_;
						ValaCCodeIdentifier* _tmp233_;
						ValaCCodeFunctionCall* _tmp234_;
						ValaCCodeFunctionCall* len_call;
						/* handle calls to methods returning null-terminated arrays*/
						_tmp230_ = NULL;
						_tmp231_ = NULL;
						temp_var = (_tmp231_ = vala_ccode_base_module_get_temp_variable ((ValaCCodeBaseModule*) self, _tmp230_ = vala_data_type_get_return_type (itype), TRUE, NULL), (_tmp230_ == NULL) ? NULL : (_tmp230_ = (vala_code_node_unref (_tmp230_), NULL)), _tmp231_);
						temp_ref = vala_ccode_base_module_get_variable_cexpression ((ValaCCodeBaseModule*) self, vala_symbol_get_name ((ValaSymbol*) temp_var));
						gee_list_insert ((GeeList*) ((ValaCCodeBaseModule*) self)->temp_vars, 0, temp_var);
						_tmp232_ = NULL;
						ccall_expr = (_tmp232_ = (ValaCCodeExpression*) vala_ccode_assignment_new (temp_ref, ccall_expr, VALA_CCODE_ASSIGNMENT_OPERATOR_SIMPLE), (ccall_expr == NULL) ? NULL : (ccall_expr = (vala_ccode_node_unref (ccall_expr), NULL)), _tmp232_);
						((ValaCCodeBaseModule*) self)->requires_array_length = TRUE;
						_tmp233_ = NULL;
						_tmp234_ = NULL;
						len_call = (_tmp234_ = vala_ccode_function_call_new ((ValaCCodeExpression*) (_tmp233_ = vala_ccode_identifier_new ("_vala_array_length"))), (_tmp233_ == NULL) ? NULL : (_tmp233_ = (vala_ccode_node_unref (_tmp233_), NULL)), _tmp234_);
						vala_ccode_function_call_add_argument (len_call, temp_ref);
						vala_expression_append_array_size ((ValaExpression*) expr, (ValaCCodeExpression*) len_call);
						(temp_var == NULL) ? NULL : (temp_var = (vala_code_node_unref (temp_var), NULL));
						(temp_ref == NULL) ? NULL : (temp_ref = (vala_ccode_node_unref (temp_ref), NULL));
						(len_call == NULL) ? NULL : (len_call = (vala_ccode_node_unref (len_call), NULL));
					} else {
						if (!vala_method_get_no_array_length (m)) {
							ValaLocalVariable* temp_var;
							ValaCCodeExpression* temp_ref;
							ValaCCodeUnaryExpression* _tmp235_;
							temp_var = vala_ccode_base_module_get_temp_variable ((ValaCCodeBaseModule*) self, ((ValaCCodeBaseModule*) self)->int_type, TRUE, NULL);
							temp_ref = vala_ccode_base_module_get_variable_cexpression ((ValaCCodeBaseModule*) self, vala_symbol_get_name ((ValaSymbol*) temp_var));
							gee_list_insert ((GeeList*) ((ValaCCodeBaseModule*) self)->temp_vars, 0, temp_var);
							_tmp235_ = NULL;
							gee_map_set ((GeeMap*) out_arg_map, GINT_TO_POINTER (vala_ccode_base_module_get_param_pos ((ValaCCodeBaseModule*) self, vala_method_get_carray_length_parameter_position (m) + (0.01 * dim), FALSE)), (ValaCCodeExpression*) (_tmp235_ = vala_ccode_unary_expression_new (VALA_CCODE_UNARY_OPERATOR_ADDRESS_OF, temp_ref)));
							(_tmp235_ == NULL) ? NULL : (_tmp235_ = (vala_ccode_node_unref (_tmp235_), NULL));
							vala_expression_append_array_size ((ValaExpression*) expr, temp_ref);
							(temp_var == NULL) ? NULL : (temp_var = (vala_code_node_unref (temp_var), NULL));
							(temp_ref == NULL) ? NULL : (temp_ref = (vala_ccode_node_unref (temp_ref), NULL));
						} else {
							ValaCCodeConstant* _tmp236_;
							_tmp236_ = NULL;
							vala_expression_append_array_size ((ValaExpression*) expr, (ValaCCodeExpression*) (_tmp236_ = vala_ccode_constant_new ("-1")));
							(_tmp236_ == NULL) ? NULL : (_tmp236_ = (vala_ccode_node_unref (_tmp236_), NULL));
						}
					}
				}
			}
		}
		(array_type == NULL) ? NULL : (array_type = (vala_code_node_unref (array_type), NULL));
	} else {
		gboolean _tmp237_;
		_tmp237_ = FALSE;
		if (m != NULL) {
			_tmp237_ = VALA_IS_DELEGATE_TYPE (vala_method_get_return_type (m));
		} else {
			_tmp237_ = FALSE;
		}
		if (_tmp237_) {
			ValaDelegateType* _tmp238_;
			ValaDelegateType* deleg_type;
			ValaDelegate* _tmp239_;
			ValaDelegate* d;
			_tmp238_ = NULL;
			deleg_type = (_tmp238_ = VALA_DELEGATE_TYPE (vala_method_get_return_type (m)), (_tmp238_ == NULL) ? NULL : vala_code_node_ref (_tmp238_));
			_tmp239_ = NULL;
			d = (_tmp239_ = vala_delegate_type_get_delegate_symbol (deleg_type), (_tmp239_ == NULL) ? NULL : vala_code_node_ref (_tmp239_));
			if (vala_delegate_get_has_target (d)) {
				ValaPointerType* _tmp241_;
				ValaVoidType* _tmp240_;
				ValaLocalVariable* _tmp242_;
				ValaLocalVariable* temp_var;
				ValaCCodeExpression* temp_ref;
				ValaCCodeUnaryExpression* _tmp243_;
				_tmp241_ = NULL;
				_tmp240_ = NULL;
				_tmp242_ = NULL;
				temp_var = (_tmp242_ = vala_ccode_base_module_get_temp_variable ((ValaCCodeBaseModule*) self, (ValaDataType*) (_tmp241_ = vala_pointer_type_new ((ValaDataType*) (_tmp240_ = vala_void_type_new (NULL)), NULL)), TRUE, NULL), (_tmp241_ == NULL) ? NULL : (_tmp241_ = (vala_code_node_unref (_tmp241_), NULL)), (_tmp240_ == NULL) ? NULL : (_tmp240_ = (vala_code_node_unref (_tmp240_), NULL)), _tmp242_);
				temp_ref = vala_ccode_base_module_get_variable_cexpression ((ValaCCodeBaseModule*) self, vala_symbol_get_name ((ValaSymbol*) temp_var));
				gee_list_insert ((GeeList*) ((ValaCCodeBaseModule*) self)->temp_vars, 0, temp_var);
				_tmp243_ = NULL;
				gee_map_set ((GeeMap*) out_arg_map, GINT_TO_POINTER (vala_ccode_base_module_get_param_pos ((ValaCCodeBaseModule*) self, vala_method_get_cdelegate_target_parameter_position (m), FALSE)), (ValaCCodeExpression*) (_tmp243_ = vala_ccode_unary_expression_new (VALA_CCODE_UNARY_OPERATOR_ADDRESS_OF, temp_ref)));
				(_tmp243_ == NULL) ? NULL : (_tmp243_ = (vala_ccode_node_unref (_tmp243_), NULL));
				vala_method_call_set_delegate_target (expr, temp_ref);
				(temp_var == NULL) ? NULL : (temp_var = (vala_code_node_unref (temp_var), NULL));
				(temp_ref == NULL) ? NULL : (temp_ref = (vala_ccode_node_unref (temp_ref), NULL));
			}
			(deleg_type == NULL) ? NULL : (deleg_type = (vala_code_node_unref (deleg_type), NULL));
			(d == NULL) ? NULL : (d = (vala_code_node_unref (d), NULL));
		}
	}
	_tmp244_ = FALSE;
	if (deleg != NULL) {
		_tmp244_ = VALA_IS_ARRAY_TYPE (vala_delegate_get_return_type (deleg));
	} else {
		_tmp244_ = FALSE;
	}
	/* add length argument for delegates returning arrays
	 TODO: avoid code duplication with methods returning arrays, see above*/
	if (_tmp244_) {
		ValaArrayType* _tmp245_;
		ValaArrayType* array_type;
		_tmp245_ = NULL;
		array_type = (_tmp245_ = VALA_ARRAY_TYPE (vala_delegate_get_return_type (deleg)), (_tmp245_ == NULL) ? NULL : vala_code_node_ref (_tmp245_));
		{
			gint dim;
			dim = 1;
			{
				gboolean _tmp246_;
				_tmp246_ = TRUE;
				while (TRUE) {
					if (!_tmp246_) {
						dim++;
					}
					_tmp246_ = FALSE;
					if (!(dim <= vala_array_type_get_rank (array_type))) {
						break;
					}
					if (vala_delegate_get_array_null_terminated (deleg)) {
						ValaDataType* _tmp247_;
						ValaLocalVariable* _tmp248_;
						ValaLocalVariable* temp_var;
						ValaCCodeExpression* temp_ref;
						ValaCCodeExpression* _tmp249_;
						ValaCCodeIdentifier* _tmp250_;
						ValaCCodeFunctionCall* _tmp251_;
						ValaCCodeFunctionCall* len_call;
						/* handle calls to methods returning null-terminated arrays*/
						_tmp247_ = NULL;
						_tmp248_ = NULL;
						temp_var = (_tmp248_ = vala_ccode_base_module_get_temp_variable ((ValaCCodeBaseModule*) self, _tmp247_ = vala_data_type_get_return_type (itype), TRUE, NULL), (_tmp247_ == NULL) ? NULL : (_tmp247_ = (vala_code_node_unref (_tmp247_), NULL)), _tmp248_);
						temp_ref = vala_ccode_base_module_get_variable_cexpression ((ValaCCodeBaseModule*) self, vala_symbol_get_name ((ValaSymbol*) temp_var));
						gee_list_insert ((GeeList*) ((ValaCCodeBaseModule*) self)->temp_vars, 0, temp_var);
						_tmp249_ = NULL;
						ccall_expr = (_tmp249_ = (ValaCCodeExpression*) vala_ccode_assignment_new (temp_ref, ccall_expr, VALA_CCODE_ASSIGNMENT_OPERATOR_SIMPLE), (ccall_expr == NULL) ? NULL : (ccall_expr = (vala_ccode_node_unref (ccall_expr), NULL)), _tmp249_);
						((ValaCCodeBaseModule*) self)->requires_array_length = TRUE;
						_tmp250_ = NULL;
						_tmp251_ = NULL;
						len_call = (_tmp251_ = vala_ccode_function_call_new ((ValaCCodeExpression*) (_tmp250_ = vala_ccode_identifier_new ("_vala_array_length"))), (_tmp250_ == NULL) ? NULL : (_tmp250_ = (vala_ccode_node_unref (_tmp250_), NULL)), _tmp251_);
						vala_ccode_function_call_add_argument (len_call, temp_ref);
						vala_expression_append_array_size ((ValaExpression*) expr, (ValaCCodeExpression*) len_call);
						(temp_var == NULL) ? NULL : (temp_var = (vala_code_node_unref (temp_var), NULL));
						(temp_ref == NULL) ? NULL : (temp_ref = (vala_ccode_node_unref (temp_ref), NULL));
						(len_call == NULL) ? NULL : (len_call = (vala_ccode_node_unref (len_call), NULL));
					} else {
						if (!vala_delegate_get_no_array_length (deleg)) {
							ValaLocalVariable* temp_var;
							ValaCCodeExpression* temp_ref;
							ValaCCodeUnaryExpression* _tmp252_;
							temp_var = vala_ccode_base_module_get_temp_variable ((ValaCCodeBaseModule*) self, ((ValaCCodeBaseModule*) self)->int_type, TRUE, NULL);
							temp_ref = vala_ccode_base_module_get_variable_cexpression ((ValaCCodeBaseModule*) self, vala_symbol_get_name ((ValaSymbol*) temp_var));
							gee_list_insert ((GeeList*) ((ValaCCodeBaseModule*) self)->temp_vars, 0, temp_var);
							_tmp252_ = NULL;
							gee_map_set ((GeeMap*) out_arg_map, GINT_TO_POINTER (vala_ccode_base_module_get_param_pos ((ValaCCodeBaseModule*) self, vala_delegate_get_carray_length_parameter_position (deleg) + (0.01 * dim), FALSE)), (ValaCCodeExpression*) (_tmp252_ = vala_ccode_unary_expression_new (VALA_CCODE_UNARY_OPERATOR_ADDRESS_OF, temp_ref)));
							(_tmp252_ == NULL) ? NULL : (_tmp252_ = (vala_ccode_node_unref (_tmp252_), NULL));
							vala_expression_append_array_size ((ValaExpression*) expr, temp_ref);
							(temp_var == NULL) ? NULL : (temp_var = (vala_code_node_unref (temp_var), NULL));
							(temp_ref == NULL) ? NULL : (temp_ref = (vala_ccode_node_unref (temp_ref), NULL));
						} else {
							ValaCCodeConstant* _tmp253_;
							_tmp253_ = NULL;
							vala_expression_append_array_size ((ValaExpression*) expr, (ValaCCodeExpression*) (_tmp253_ = vala_ccode_constant_new ("-1")));
							(_tmp253_ == NULL) ? NULL : (_tmp253_ = (vala_ccode_node_unref (_tmp253_), NULL));
						}
					}
				}
			}
		}
		(array_type == NULL) ? NULL : (array_type = (vala_code_node_unref (array_type), NULL));
	} else {
		gboolean _tmp254_;
		_tmp254_ = FALSE;
		if (deleg != NULL) {
			_tmp254_ = VALA_IS_DELEGATE_TYPE (vala_delegate_get_return_type (deleg));
		} else {
			_tmp254_ = FALSE;
		}
		if (_tmp254_) {
			ValaDelegateType* _tmp255_;
			ValaDelegateType* deleg_type;
			ValaDelegate* _tmp256_;
			ValaDelegate* d;
			_tmp255_ = NULL;
			deleg_type = (_tmp255_ = VALA_DELEGATE_TYPE (vala_delegate_get_return_type (deleg)), (_tmp255_ == NULL) ? NULL : vala_code_node_ref (_tmp255_));
			_tmp256_ = NULL;
			d = (_tmp256_ = vala_delegate_type_get_delegate_symbol (deleg_type), (_tmp256_ == NULL) ? NULL : vala_code_node_ref (_tmp256_));
			if (vala_delegate_get_has_target (d)) {
				ValaPointerType* _tmp258_;
				ValaVoidType* _tmp257_;
				ValaLocalVariable* _tmp259_;
				ValaLocalVariable* temp_var;
				ValaCCodeExpression* temp_ref;
				ValaCCodeUnaryExpression* _tmp260_;
				_tmp258_ = NULL;
				_tmp257_ = NULL;
				_tmp259_ = NULL;
				temp_var = (_tmp259_ = vala_ccode_base_module_get_temp_variable ((ValaCCodeBaseModule*) self, (ValaDataType*) (_tmp258_ = vala_pointer_type_new ((ValaDataType*) (_tmp257_ = vala_void_type_new (NULL)), NULL)), TRUE, NULL), (_tmp258_ == NULL) ? NULL : (_tmp258_ = (vala_code_node_unref (_tmp258_), NULL)), (_tmp257_ == NULL) ? NULL : (_tmp257_ = (vala_code_node_unref (_tmp257_), NULL)), _tmp259_);
				temp_ref = vala_ccode_base_module_get_variable_cexpression ((ValaCCodeBaseModule*) self, vala_symbol_get_name ((ValaSymbol*) temp_var));
				gee_list_insert ((GeeList*) ((ValaCCodeBaseModule*) self)->temp_vars, 0, temp_var);
				_tmp260_ = NULL;
				gee_map_set ((GeeMap*) out_arg_map, GINT_TO_POINTER (vala_ccode_base_module_get_param_pos ((ValaCCodeBaseModule*) self, vala_delegate_get_cdelegate_target_parameter_position (deleg), FALSE)), (ValaCCodeExpression*) (_tmp260_ = vala_ccode_unary_expression_new (VALA_CCODE_UNARY_OPERATOR_ADDRESS_OF, temp_ref)));
				(_tmp260_ == NULL) ? NULL : (_tmp260_ = (vala_ccode_node_unref (_tmp260_), NULL));
				vala_method_call_set_delegate_target (expr, temp_ref);
				(temp_var == NULL) ? NULL : (temp_var = (vala_code_node_unref (temp_var), NULL));
				(temp_ref == NULL) ? NULL : (temp_ref = (vala_ccode_node_unref (temp_ref), NULL));
			}
			(deleg_type == NULL) ? NULL : (deleg_type = (vala_code_node_unref (deleg_type), NULL));
			(d == NULL) ? NULL : (d = (vala_code_node_unref (d), NULL));
		}
	}
	_tmp261_ = FALSE;
	if (m != NULL) {
		_tmp261_ = vala_method_get_coroutine (m);
	} else {
		_tmp261_ = FALSE;
	}
	if (_tmp261_) {
		gboolean _tmp262_;
		_tmp262_ = FALSE;
		if (((ValaCCodeBaseModule*) self)->current_method != NULL) {
			_tmp262_ = vala_method_get_coroutine (((ValaCCodeBaseModule*) self)->current_method);
		} else {
			_tmp262_ = FALSE;
		}
		if (_tmp262_) {
			ValaCCodeIdentifier* _tmp265_;
			char* _tmp264_;
			char* _tmp263_;
			ValaCCodeIdentifier* _tmp266_;
			ValaCCodeMemberAccess* _tmp268_;
			ValaCCodeIdentifier* _tmp267_;
			_tmp265_ = NULL;
			_tmp264_ = NULL;
			_tmp263_ = NULL;
			gee_map_set ((GeeMap*) in_arg_map, GINT_TO_POINTER (vala_ccode_base_module_get_param_pos ((ValaCCodeBaseModule*) self, (double) (-1), FALSE)), (ValaCCodeExpression*) (_tmp265_ = vala_ccode_identifier_new (_tmp264_ = g_strconcat (_tmp263_ = vala_method_get_cname (((ValaCCodeBaseModule*) self)->current_method), "_ready", NULL))));
			(_tmp265_ == NULL) ? NULL : (_tmp265_ = (vala_ccode_node_unref (_tmp265_), NULL));
			_tmp264_ = (g_free (_tmp264_), NULL);
			_tmp263_ = (g_free (_tmp263_), NULL);
			_tmp266_ = NULL;
			gee_map_set ((GeeMap*) in_arg_map, GINT_TO_POINTER (vala_ccode_base_module_get_param_pos ((ValaCCodeBaseModule*) self, -0.9, FALSE)), (ValaCCodeExpression*) (_tmp266_ = vala_ccode_identifier_new ("data")));
			(_tmp266_ == NULL) ? NULL : (_tmp266_ = (vala_ccode_node_unref (_tmp266_), NULL));
			/* pass GAsyncResult stored in closure to finish function*/
			_tmp268_ = NULL;
			_tmp267_ = NULL;
			gee_map_set ((GeeMap*) out_arg_map, GINT_TO_POINTER (vala_ccode_base_module_get_param_pos ((ValaCCodeBaseModule*) self, 0.1, FALSE)), (ValaCCodeExpression*) (_tmp268_ = vala_ccode_member_access_new_pointer ((ValaCCodeExpression*) (_tmp267_ = vala_ccode_identifier_new ("data")), "res")));
			(_tmp268_ == NULL) ? NULL : (_tmp268_ = (vala_ccode_node_unref (_tmp268_), NULL));
			(_tmp267_ == NULL) ? NULL : (_tmp267_ = (vala_ccode_node_unref (_tmp267_), NULL));
		}
	}
	_tmp269_ = FALSE;
	if (VALA_IS_CREATION_METHOD (m)) {
		GeeList* _tmp270_;
		_tmp270_ = NULL;
		_tmp269_ = gee_collection_get_size ((GeeCollection*) (_tmp270_ = vala_code_node_get_error_types ((ValaCodeNode*) m))) > 0;
		(_tmp270_ == NULL) ? NULL : (_tmp270_ = (gee_collection_object_unref (_tmp270_), NULL));
	} else {
		_tmp269_ = FALSE;
	}
	if (_tmp269_) {
		ValaCCodeIdentifier* _tmp271_;
		_tmp271_ = NULL;
		gee_map_set ((GeeMap*) out_arg_map, GINT_TO_POINTER (vala_ccode_base_module_get_param_pos ((ValaCCodeBaseModule*) self, (double) (-1), FALSE)), (ValaCCodeExpression*) (_tmp271_ = vala_ccode_identifier_new ("error")));
		(_tmp271_ == NULL) ? NULL : (_tmp271_ = (vala_ccode_node_unref (_tmp271_), NULL));
	} else {
		if (vala_code_node_get_tree_can_fail ((ValaCodeNode*) expr)) {
			ValaCCodeUnaryExpression* _tmp273_;
			ValaCCodeExpression* _tmp272_;
			/* method can fail*/
			((ValaCCodeBaseModule*) self)->current_method_inner_error = TRUE;
			/* add &inner_error before the ellipsis arguments*/
			_tmp273_ = NULL;
			_tmp272_ = NULL;
			gee_map_set ((GeeMap*) out_arg_map, GINT_TO_POINTER (vala_ccode_base_module_get_param_pos ((ValaCCodeBaseModule*) self, (double) (-1), FALSE)), (ValaCCodeExpression*) (_tmp273_ = vala_ccode_unary_expression_new (VALA_CCODE_UNARY_OPERATOR_ADDRESS_OF, _tmp272_ = vala_ccode_base_module_get_variable_cexpression ((ValaCCodeBaseModule*) self, "_inner_error_"))));
			(_tmp273_ == NULL) ? NULL : (_tmp273_ = (vala_ccode_node_unref (_tmp273_), NULL));
			(_tmp272_ == NULL) ? NULL : (_tmp272_ = (vala_ccode_node_unref (_tmp272_), NULL));
		}
	}
	if (ellipsis) {
		gboolean _tmp274_;
		_tmp274_ = FALSE;
		if (!vala_method_get_printf_format (m)) {
			_tmp274_ = _vala_strcmp0 (vala_method_get_sentinel (m), "") != 0;
		} else {
			_tmp274_ = FALSE;
		}
		/* ensure variable argument list ends with NULL
		 * except when using printf-style arguments */
		if (_tmp274_) {
			ValaCCodeConstant* _tmp275_;
			_tmp275_ = NULL;
			gee_map_set ((GeeMap*) in_arg_map, GINT_TO_POINTER (vala_ccode_base_module_get_param_pos ((ValaCCodeBaseModule*) self, (double) (-1), TRUE)), (ValaCCodeExpression*) (_tmp275_ = vala_ccode_constant_new (vala_method_get_sentinel (m))));
			(_tmp275_ == NULL) ? NULL : (_tmp275_ = (vala_ccode_node_unref (_tmp275_), NULL));
		}
	} else {
		if (VALA_IS_DELEGATE_TYPE (itype)) {
			ValaDelegateType* _tmp276_;
			ValaDelegateType* deleg_type;
			ValaDelegate* _tmp277_;
			ValaDelegate* d;
			_tmp276_ = NULL;
			deleg_type = (_tmp276_ = VALA_DELEGATE_TYPE (itype), (_tmp276_ == NULL) ? NULL : vala_code_node_ref (_tmp276_));
			_tmp277_ = NULL;
			d = (_tmp277_ = vala_delegate_type_get_delegate_symbol (deleg_type), (_tmp277_ == NULL) ? NULL : vala_code_node_ref (_tmp277_));
			if (vala_delegate_get_has_target (d)) {
				ValaCCodeExpression* _tmp278_;
				ValaCCodeExpression* _tmp279_;
				_tmp278_ = NULL;
				gee_map_set ((GeeMap*) in_arg_map, GINT_TO_POINTER (vala_ccode_base_module_get_param_pos ((ValaCCodeBaseModule*) self, vala_delegate_get_cinstance_parameter_position (d), FALSE)), _tmp278_ = vala_ccode_base_module_get_delegate_target_cexpression ((ValaCCodeBaseModule*) self, vala_method_call_get_call (expr)));
				(_tmp278_ == NULL) ? NULL : (_tmp278_ = (vala_ccode_node_unref (_tmp278_), NULL));
				_tmp279_ = NULL;
				gee_map_set ((GeeMap*) out_arg_map, GINT_TO_POINTER (vala_ccode_base_module_get_param_pos ((ValaCCodeBaseModule*) self, vala_delegate_get_cinstance_parameter_position (d), FALSE)), _tmp279_ = vala_ccode_base_module_get_delegate_target_cexpression ((ValaCCodeBaseModule*) self, vala_method_call_get_call (expr)));
				(_tmp279_ == NULL) ? NULL : (_tmp279_ = (vala_ccode_node_unref (_tmp279_), NULL));
			}
			(deleg_type == NULL) ? NULL : (deleg_type = (vala_code_node_unref (deleg_type), NULL));
			(d == NULL) ? NULL : (d = (vala_code_node_unref (d), NULL));
		}
	}
	_tmp280_ = FALSE;
	if (VALA_IS_SIGNAL_TYPE (itype)) {
		ValaDataType* _tmp281_;
		_tmp281_ = NULL;
		_tmp280_ = !VALA_IS_VOID_TYPE (_tmp281_ = vala_data_type_get_return_type (itype));
		(_tmp281_ == NULL) ? NULL : (_tmp281_ = (vala_code_node_unref (_tmp281_), NULL));
	} else {
		_tmp280_ = FALSE;
	}
	/* pass address for the return value of non-void signals without emitter functions*/
	if (_tmp280_) {
		ValaSignal* _tmp282_;
		ValaSignal* sig;
		gboolean _tmp283_;
		gboolean _tmp284_;
		_tmp282_ = NULL;
		sig = (_tmp282_ = vala_signal_type_get_signal_symbol (VALA_SIGNAL_TYPE (itype)), (_tmp282_ == NULL) ? NULL : vala_code_node_ref (_tmp282_));
		_tmp283_ = FALSE;
		_tmp284_ = FALSE;
		if (ma != NULL) {
			_tmp284_ = VALA_IS_BASE_ACCESS (vala_member_access_get_inner (ma));
		} else {
			_tmp284_ = FALSE;
		}
		if (_tmp284_) {
			_tmp283_ = vala_signal_get_is_virtual (sig);
		} else {
			_tmp283_ = FALSE;
		}
		if (_tmp283_) {
		} else {
			if (!vala_signal_get_has_emitter (sig)) {
				ValaDataType* _tmp285_;
				ValaLocalVariable* _tmp286_;
				ValaLocalVariable* temp_var;
				ValaCCodeExpression* temp_ref;
				ValaCCodeUnaryExpression* _tmp287_;
				ValaCCodeCommaExpression* ccomma;
				ValaCCodeExpression* _tmp289_;
				ValaCCodeExpression* _tmp288_;
				_tmp285_ = NULL;
				_tmp286_ = NULL;
				temp_var = (_tmp286_ = vala_ccode_base_module_get_temp_variable ((ValaCCodeBaseModule*) self, _tmp285_ = vala_data_type_get_return_type (itype), TRUE, NULL), (_tmp285_ == NULL) ? NULL : (_tmp285_ = (vala_code_node_unref (_tmp285_), NULL)), _tmp286_);
				temp_ref = vala_ccode_base_module_get_variable_cexpression ((ValaCCodeBaseModule*) self, vala_symbol_get_name ((ValaSymbol*) temp_var));
				gee_list_insert ((GeeList*) ((ValaCCodeBaseModule*) self)->temp_vars, 0, temp_var);
				_tmp287_ = NULL;
				gee_map_set ((GeeMap*) out_arg_map, GINT_TO_POINTER (vala_ccode_base_module_get_param_pos ((ValaCCodeBaseModule*) self, (double) (-1), TRUE)), (ValaCCodeExpression*) (_tmp287_ = vala_ccode_unary_expression_new (VALA_CCODE_UNARY_OPERATOR_ADDRESS_OF, temp_ref)));
				(_tmp287_ == NULL) ? NULL : (_tmp287_ = (vala_ccode_node_unref (_tmp287_), NULL));
				ccomma = vala_ccode_comma_expression_new ();
				vala_ccode_comma_expression_append_expression (ccomma, VALA_CCODE_EXPRESSION (ccall_expr));
				vala_ccode_comma_expression_append_expression (ccomma, VALA_CCODE_EXPRESSION (temp_ref));
				_tmp289_ = NULL;
				_tmp288_ = NULL;
				ccall_expr = (_tmp289_ = (_tmp288_ = (ValaCCodeExpression*) ccomma, (_tmp288_ == NULL) ? NULL : vala_ccode_node_ref (_tmp288_)), (ccall_expr == NULL) ? NULL : (ccall_expr = (vala_ccode_node_unref (ccall_expr), NULL)), _tmp289_);
				(temp_var == NULL) ? NULL : (temp_var = (vala_code_node_unref (temp_var), NULL));
				(temp_ref == NULL) ? NULL : (temp_ref = (vala_ccode_node_unref (temp_ref), NULL));
				(ccomma == NULL) ? NULL : (ccomma = (vala_ccode_node_unref (ccomma), NULL));
			}
		}
		(sig == NULL) ? NULL : (sig = (vala_code_node_unref (sig), NULL));
	}
	/* append C arguments in the right order*/
	last_pos = 0;
	min_pos = 0;
	if (async_call != ccall) {
		/* don't append out arguments for .begin() calls*/
		last_pos = -1;
		while (TRUE) {
			ValaCCodeExpression* _tmp294_;
			min_pos = -1;
			{
				GeeSet* _tmp290_;
				GeeIterator* _tmp291_;
				GeeIterator* _pos_it;
				_tmp290_ = NULL;
				_tmp291_ = NULL;
				_pos_it = (_tmp291_ = gee_iterable_iterator ((GeeIterable*) (_tmp290_ = gee_map_get_keys ((GeeMap*) out_arg_map))), (_tmp290_ == NULL) ? NULL : (_tmp290_ = (gee_collection_object_unref (_tmp290_), NULL)), _tmp291_);
				while (TRUE) {
					gint pos;
					gboolean _tmp292_;
					if (!gee_iterator_next (_pos_it)) {
						break;
					}
					pos = GPOINTER_TO_INT (gee_iterator_get (_pos_it));
					_tmp292_ = FALSE;
					if (pos > last_pos) {
						gboolean _tmp293_;
						_tmp293_ = FALSE;
						if (min_pos == (-1)) {
							_tmp293_ = TRUE;
						} else {
							_tmp293_ = pos < min_pos;
						}
						_tmp292_ = _tmp293_;
					} else {
						_tmp292_ = FALSE;
					}
					if (_tmp292_) {
						min_pos = pos;
					}
				}
				(_pos_it == NULL) ? NULL : (_pos_it = (gee_collection_object_unref (_pos_it), NULL));
			}
			if (min_pos == (-1)) {
				break;
			}
			_tmp294_ = NULL;
			vala_ccode_function_call_add_argument (ccall, _tmp294_ = (ValaCCodeExpression*) gee_map_get ((GeeMap*) out_arg_map, GINT_TO_POINTER (min_pos)));
			(_tmp294_ == NULL) ? NULL : (_tmp294_ = (vala_ccode_node_unref (_tmp294_), NULL));
			last_pos = min_pos;
		}
	}
	if (async_call != NULL) {
		last_pos = -1;
		while (TRUE) {
			ValaCCodeExpression* _tmp299_;
			min_pos = -1;
			{
				GeeSet* _tmp295_;
				GeeIterator* _tmp296_;
				GeeIterator* _pos_it;
				_tmp295_ = NULL;
				_tmp296_ = NULL;
				_pos_it = (_tmp296_ = gee_iterable_iterator ((GeeIterable*) (_tmp295_ = gee_map_get_keys ((GeeMap*) in_arg_map))), (_tmp295_ == NULL) ? NULL : (_tmp295_ = (gee_collection_object_unref (_tmp295_), NULL)), _tmp296_);
				while (TRUE) {
					gint pos;
					gboolean _tmp297_;
					if (!gee_iterator_next (_pos_it)) {
						break;
					}
					pos = GPOINTER_TO_INT (gee_iterator_get (_pos_it));
					_tmp297_ = FALSE;
					if (pos > last_pos) {
						gboolean _tmp298_;
						_tmp298_ = FALSE;
						if (min_pos == (-1)) {
							_tmp298_ = TRUE;
						} else {
							_tmp298_ = pos < min_pos;
						}
						_tmp297_ = _tmp298_;
					} else {
						_tmp297_ = FALSE;
					}
					if (_tmp297_) {
						min_pos = pos;
					}
				}
				(_pos_it == NULL) ? NULL : (_pos_it = (gee_collection_object_unref (_pos_it), NULL));
			}
			if (min_pos == (-1)) {
				break;
			}
			_tmp299_ = NULL;
			vala_ccode_function_call_add_argument (async_call, _tmp299_ = (ValaCCodeExpression*) gee_map_get ((GeeMap*) in_arg_map, GINT_TO_POINTER (min_pos)));
			(_tmp299_ == NULL) ? NULL : (_tmp299_ = (vala_ccode_node_unref (_tmp299_), NULL));
			last_pos = min_pos;
		}
	}
	_tmp300_ = FALSE;
	_tmp301_ = FALSE;
	if (m != NULL) {
		_tmp301_ = vala_method_get_binding (m) == MEMBER_BINDING_INSTANCE;
	} else {
		_tmp301_ = FALSE;
	}
	if (_tmp301_) {
		_tmp300_ = vala_method_get_returns_modified_pointer (m);
	} else {
		_tmp300_ = FALSE;
	}
	if (_tmp300_) {
		ValaCCodeAssignment* _tmp302_;
		_tmp302_ = NULL;
		vala_code_node_set_ccodenode ((ValaCodeNode*) expr, (ValaCCodeNode*) (_tmp302_ = vala_ccode_assignment_new (instance, ccall_expr, VALA_CCODE_ASSIGNMENT_OPERATOR_SIMPLE)));
		(_tmp302_ == NULL) ? NULL : (_tmp302_ = (vala_ccode_node_unref (_tmp302_), NULL));
	} else {
		vala_code_node_set_ccodenode ((ValaCodeNode*) expr, (ValaCCodeNode*) ccall_expr);
	}
	_tmp303_ = FALSE;
	_tmp304_ = FALSE;
	_tmp305_ = FALSE;
	if (m != NULL) {
		_tmp305_ = vala_method_get_coroutine (m);
	} else {
		_tmp305_ = FALSE;
	}
	if (_tmp305_) {
		_tmp304_ = ((ValaCCodeBaseModule*) self)->current_method != NULL;
	} else {
		_tmp304_ = FALSE;
	}
	if (_tmp304_) {
		_tmp303_ = vala_method_get_coroutine (((ValaCCodeBaseModule*) self)->current_method);
	} else {
		_tmp303_ = FALSE;
	}
	if (_tmp303_) {
		gboolean _tmp306_;
		_tmp306_ = FALSE;
		if (_vala_strcmp0 (vala_member_access_get_member_name (ma), "begin") != 0) {
			_tmp306_ = TRUE;
		} else {
			_tmp306_ = vala_expression_get_symbol_reference (vala_member_access_get_inner (ma)) != vala_expression_get_symbol_reference ((ValaExpression*) ma);
		}
		if (_tmp306_) {
			ValaCCodeExpressionStatement* _tmp308_;
			gint state;
			ValaCCodeExpressionStatement* _tmp314_;
			ValaCCodeAssignment* _tmp313_;
			ValaCCodeConstant* _tmp312_;
			char* _tmp311_;
			ValaCCodeMemberAccess* _tmp310_;
			ValaCCodeIdentifier* _tmp309_;
			ValaCCodeReturnStatement* _tmp316_;
			ValaCCodeConstant* _tmp315_;
			ValaCCodeCaseStatement* _tmp319_;
			ValaCCodeConstant* _tmp318_;
			char* _tmp317_;
			if (((ValaCCodeBaseModule*) self)->pre_statement_fragment == NULL) {
				ValaCCodeFragment* _tmp307_;
				_tmp307_ = NULL;
				((ValaCCodeBaseModule*) self)->pre_statement_fragment = (_tmp307_ = vala_ccode_fragment_new (), (((ValaCCodeBaseModule*) self)->pre_statement_fragment == NULL) ? NULL : (((ValaCCodeBaseModule*) self)->pre_statement_fragment = (vala_ccode_node_unref (((ValaCCodeBaseModule*) self)->pre_statement_fragment), NULL)), _tmp307_);
			}
			_tmp308_ = NULL;
			vala_ccode_fragment_append (((ValaCCodeBaseModule*) self)->pre_statement_fragment, (ValaCCodeNode*) (_tmp308_ = vala_ccode_expression_statement_new ((ValaCCodeExpression*) async_call)));
			(_tmp308_ == NULL) ? NULL : (_tmp308_ = (vala_ccode_node_unref (_tmp308_), NULL));
			state = ((ValaCCodeBaseModule*) self)->next_coroutine_state++;
			_tmp314_ = NULL;
			_tmp313_ = NULL;
			_tmp312_ = NULL;
			_tmp311_ = NULL;
			_tmp310_ = NULL;
			_tmp309_ = NULL;
			vala_ccode_fragment_append (((ValaCCodeBaseModule*) self)->pre_statement_fragment, (ValaCCodeNode*) (_tmp314_ = vala_ccode_expression_statement_new ((ValaCCodeExpression*) (_tmp313_ = vala_ccode_assignment_new ((ValaCCodeExpression*) (_tmp310_ = vala_ccode_member_access_new_pointer ((ValaCCodeExpression*) (_tmp309_ = vala_ccode_identifier_new ("data")), "state")), (ValaCCodeExpression*) (_tmp312_ = vala_ccode_constant_new (_tmp311_ = g_strdup_printf ("%i", state))), VALA_CCODE_ASSIGNMENT_OPERATOR_SIMPLE)))));
			(_tmp314_ == NULL) ? NULL : (_tmp314_ = (vala_ccode_node_unref (_tmp314_), NULL));
			(_tmp313_ == NULL) ? NULL : (_tmp313_ = (vala_ccode_node_unref (_tmp313_), NULL));
			(_tmp312_ == NULL) ? NULL : (_tmp312_ = (vala_ccode_node_unref (_tmp312_), NULL));
			_tmp311_ = (g_free (_tmp311_), NULL);
			(_tmp310_ == NULL) ? NULL : (_tmp310_ = (vala_ccode_node_unref (_tmp310_), NULL));
			(_tmp309_ == NULL) ? NULL : (_tmp309_ = (vala_ccode_node_unref (_tmp309_), NULL));
			_tmp316_ = NULL;
			_tmp315_ = NULL;
			vala_ccode_fragment_append (((ValaCCodeBaseModule*) self)->pre_statement_fragment, (ValaCCodeNode*) (_tmp316_ = vala_ccode_return_statement_new ((ValaCCodeExpression*) (_tmp315_ = vala_ccode_constant_new ("FALSE")))));
			(_tmp316_ == NULL) ? NULL : (_tmp316_ = (vala_ccode_node_unref (_tmp316_), NULL));
			(_tmp315_ == NULL) ? NULL : (_tmp315_ = (vala_ccode_node_unref (_tmp315_), NULL));
			_tmp319_ = NULL;
			_tmp318_ = NULL;
			_tmp317_ = NULL;
			vala_ccode_fragment_append (((ValaCCodeBaseModule*) self)->pre_statement_fragment, (ValaCCodeNode*) (_tmp319_ = vala_ccode_case_statement_new ((ValaCCodeExpression*) (_tmp318_ = vala_ccode_constant_new (_tmp317_ = g_strdup_printf ("%i", state))))));
			(_tmp319_ == NULL) ? NULL : (_tmp319_ = (vala_ccode_node_unref (_tmp319_), NULL));
			(_tmp318_ == NULL) ? NULL : (_tmp318_ = (vala_ccode_node_unref (_tmp318_), NULL));
			_tmp317_ = (g_free (_tmp317_), NULL);
		}
	}
	if (VALA_IS_ARRAY_RESIZE_METHOD (m)) {
		GeeList* _tmp320_;
		GeeIterator* _tmp321_;
		GeeIterator* arg_it;
		ValaCCodeExpression* _tmp323_;
		ValaExpression* _tmp322_;
		ValaCCodeExpression* _tmp324_;
		ValaCCodeExpression* new_size;
		ValaLocalVariable* temp_decl;
		ValaCCodeExpression* temp_ref;
		ValaCCodeExpression* clen;
		ValaCCodeExpression* _tmp325_;
		ValaCCodeExpression* celems;
		ValaArrayType* _tmp326_;
		ValaArrayType* array_type;
		char* _tmp328_;
		char* _tmp327_;
		ValaCCodeIdentifier* _tmp329_;
		ValaCCodeIdentifier* csizeof;
		ValaCCodeBinaryExpression* cdelta;
		ValaCCodeBinaryExpression* ccheck;
		ValaCCodeIdentifier* _tmp330_;
		ValaCCodeFunctionCall* _tmp331_;
		ValaCCodeFunctionCall* czero;
		ValaCCodeBinaryExpression* _tmp332_;
		ValaCCodeConstant* _tmp333_;
		ValaCCodeBinaryExpression* _tmp334_;
		ValaCCodeCommaExpression* ccomma;
		ValaCCodeAssignment* _tmp335_;
		ValaCCodeConditionalExpression* _tmp337_;
		ValaCCodeConstant* _tmp336_;
		ValaCCodeAssignment* _tmp339_;
		ValaCCodeExpression* _tmp338_;
		/* FIXME: size expression must not be evaluated twice at runtime (potential side effects)*/
		_tmp320_ = NULL;
		_tmp321_ = NULL;
		arg_it = (_tmp321_ = gee_iterable_iterator ((GeeIterable*) (_tmp320_ = vala_method_call_get_argument_list (expr))), (_tmp320_ == NULL) ? NULL : (_tmp320_ = (gee_collection_object_unref (_tmp320_), NULL)), _tmp321_);
		gee_iterator_next (arg_it);
		_tmp323_ = NULL;
		_tmp322_ = NULL;
		_tmp324_ = NULL;
		new_size = (_tmp324_ = (_tmp323_ = VALA_CCODE_EXPRESSION (vala_code_node_get_ccodenode ((ValaCodeNode*) (_tmp322_ = (ValaExpression*) gee_iterator_get (arg_it)))), (_tmp323_ == NULL) ? NULL : vala_ccode_node_ref (_tmp323_)), (_tmp322_ == NULL) ? NULL : (_tmp322_ = (vala_code_node_unref (_tmp322_), NULL)), _tmp324_);
		temp_decl = vala_ccode_base_module_get_temp_variable ((ValaCCodeBaseModule*) self, ((ValaCCodeBaseModule*) self)->int_type, TRUE, NULL);
		temp_ref = vala_ccode_base_module_get_variable_cexpression ((ValaCCodeBaseModule*) self, vala_symbol_get_name ((ValaSymbol*) temp_decl));
		gee_list_insert ((GeeList*) ((ValaCCodeBaseModule*) self)->temp_vars, 0, temp_decl);
		/* memset needs string.h */
		vala_ccode_declaration_space_add_include (((ValaCCodeBaseModule*) self)->source_declarations, "string.h", FALSE);
		clen = vala_ccode_module_get_array_length_cexpression (vala_ccode_module_get_head ((ValaCCodeModule*) self), vala_member_access_get_inner (ma), 1);
		_tmp325_ = NULL;
		celems = (_tmp325_ = VALA_CCODE_EXPRESSION (vala_code_node_get_ccodenode ((ValaCodeNode*) vala_member_access_get_inner (ma))), (_tmp325_ == NULL) ? NULL : vala_ccode_node_ref (_tmp325_));
		_tmp326_ = NULL;
		array_type = (_tmp326_ = VALA_ARRAY_TYPE (vala_expression_get_value_type (vala_member_access_get_inner (ma))), (_tmp326_ == NULL) ? NULL : vala_code_node_ref (_tmp326_));
		_tmp328_ = NULL;
		_tmp327_ = NULL;
		_tmp329_ = NULL;
		csizeof = (_tmp329_ = vala_ccode_identifier_new (_tmp328_ = g_strdup_printf ("sizeof (%s)", _tmp327_ = vala_data_type_get_cname (vala_array_type_get_element_type (array_type)))), _tmp328_ = (g_free (_tmp328_), NULL), _tmp327_ = (g_free (_tmp327_), NULL), _tmp329_);
		cdelta = vala_ccode_binary_expression_new (VALA_CCODE_BINARY_OPERATOR_MINUS, temp_ref, clen);
		ccheck = vala_ccode_binary_expression_new (VALA_CCODE_BINARY_OPERATOR_GREATER_THAN, temp_ref, clen);
		_tmp330_ = NULL;
		_tmp331_ = NULL;
		czero = (_tmp331_ = vala_ccode_function_call_new ((ValaCCodeExpression*) (_tmp330_ = vala_ccode_identifier_new ("memset"))), (_tmp330_ == NULL) ? NULL : (_tmp330_ = (vala_ccode_node_unref (_tmp330_), NULL)), _tmp331_);
		_tmp332_ = NULL;
		vala_ccode_function_call_add_argument (czero, (ValaCCodeExpression*) (_tmp332_ = vala_ccode_binary_expression_new (VALA_CCODE_BINARY_OPERATOR_PLUS, celems, clen)));
		(_tmp332_ == NULL) ? NULL : (_tmp332_ = (vala_ccode_node_unref (_tmp332_), NULL));
		_tmp333_ = NULL;
		vala_ccode_function_call_add_argument (czero, (ValaCCodeExpression*) (_tmp333_ = vala_ccode_constant_new ("0")));
		(_tmp333_ == NULL) ? NULL : (_tmp333_ = (vala_ccode_node_unref (_tmp333_), NULL));
		_tmp334_ = NULL;
		vala_ccode_function_call_add_argument (czero, (ValaCCodeExpression*) (_tmp334_ = vala_ccode_binary_expression_new (VALA_CCODE_BINARY_OPERATOR_MUL, (ValaCCodeExpression*) csizeof, (ValaCCodeExpression*) cdelta)));
		(_tmp334_ == NULL) ? NULL : (_tmp334_ = (vala_ccode_node_unref (_tmp334_), NULL));
		ccomma = vala_ccode_comma_expression_new ();
		_tmp335_ = NULL;
		vala_ccode_comma_expression_append_expression (ccomma, (ValaCCodeExpression*) (_tmp335_ = vala_ccode_assignment_new (temp_ref, new_size, VALA_CCODE_ASSIGNMENT_OPERATOR_SIMPLE)));
		(_tmp335_ == NULL) ? NULL : (_tmp335_ = (vala_ccode_node_unref (_tmp335_), NULL));
		vala_ccode_comma_expression_append_expression (ccomma, VALA_CCODE_EXPRESSION (vala_code_node_get_ccodenode ((ValaCodeNode*) expr)));
		_tmp337_ = NULL;
		_tmp336_ = NULL;
		vala_ccode_comma_expression_append_expression (ccomma, (ValaCCodeExpression*) (_tmp337_ = vala_ccode_conditional_expression_new ((ValaCCodeExpression*) ccheck, (ValaCCodeExpression*) czero, (ValaCCodeExpression*) (_tmp336_ = vala_ccode_constant_new ("NULL")))));
		(_tmp337_ == NULL) ? NULL : (_tmp337_ = (vala_ccode_node_unref (_tmp337_), NULL));
		(_tmp336_ == NULL) ? NULL : (_tmp336_ = (vala_ccode_node_unref (_tmp336_), NULL));
		_tmp339_ = NULL;
		_tmp338_ = NULL;
		vala_ccode_comma_expression_append_expression (ccomma, (ValaCCodeExpression*) (_tmp339_ = vala_ccode_assignment_new (_tmp338_ = vala_ccode_module_get_array_length_cexpression (vala_ccode_module_get_head ((ValaCCodeModule*) self), vala_member_access_get_inner (ma), 1), temp_ref, VALA_CCODE_ASSIGNMENT_OPERATOR_SIMPLE)));
		(_tmp339_ == NULL) ? NULL : (_tmp339_ = (vala_ccode_node_unref (_tmp339_), NULL));
		(_tmp338_ == NULL) ? NULL : (_tmp338_ = (vala_ccode_node_unref (_tmp338_), NULL));
		vala_code_node_set_ccodenode ((ValaCodeNode*) expr, (ValaCCodeNode*) ccomma);
		(arg_it == NULL) ? NULL : (arg_it = (gee_collection_object_unref (arg_it), NULL));
		(new_size == NULL) ? NULL : (new_size = (vala_ccode_node_unref (new_size), NULL));
		(temp_decl == NULL) ? NULL : (temp_decl = (vala_code_node_unref (temp_decl), NULL));
		(temp_ref == NULL) ? NULL : (temp_ref = (vala_ccode_node_unref (temp_ref), NULL));
		(clen == NULL) ? NULL : (clen = (vala_ccode_node_unref (clen), NULL));
		(celems == NULL) ? NULL : (celems = (vala_ccode_node_unref (celems), NULL));
		(array_type == NULL) ? NULL : (array_type = (vala_code_node_unref (array_type), NULL));
		(csizeof == NULL) ? NULL : (csizeof = (vala_ccode_node_unref (csizeof), NULL));
		(cdelta == NULL) ? NULL : (cdelta = (vala_ccode_node_unref (cdelta), NULL));
		(ccheck == NULL) ? NULL : (ccheck = (vala_ccode_node_unref (ccheck), NULL));
		(czero == NULL) ? NULL : (czero = (vala_ccode_node_unref (czero), NULL));
		(ccomma == NULL) ? NULL : (ccomma = (vala_ccode_node_unref (ccomma), NULL));
	}
	(ccall == NULL) ? NULL : (ccall = (vala_ccode_node_unref (ccall), NULL));
	(async_call == NULL) ? NULL : (async_call = (vala_ccode_node_unref (async_call), NULL));
	(m == NULL) ? NULL : (m = (vala_code_node_unref (m), NULL));
	(deleg == NULL) ? NULL : (deleg = (vala_code_node_unref (deleg), NULL));
	(params == NULL) ? NULL : (params = (gee_collection_object_unref (params), NULL));
	(ma == NULL) ? NULL : (ma = (vala_code_node_unref (ma), NULL));
	(itype == NULL) ? NULL : (itype = (vala_code_node_unref (itype), NULL));
	(in_arg_map == NULL) ? NULL : (in_arg_map = (gee_collection_object_unref (in_arg_map), NULL));
	(out_arg_map == NULL) ? NULL : (out_arg_map = (gee_collection_object_unref (out_arg_map), NULL));
	(ccall_expr == NULL) ? NULL : (ccall_expr = (vala_ccode_node_unref (ccall_expr), NULL));
	(instance == NULL) ? NULL : (instance = (vala_ccode_node_unref (instance), NULL));
	(params_it == NULL) ? NULL : (params_it = (gee_collection_object_unref (params_it), NULL));
}


static void vala_ccode_method_call_module_add_generic_type_arguments (ValaCCodeMethodCallModule* self, ValaCCodeFunctionCall* ccall, GeeList* type_args, ValaCodeNode* expr) {
	g_return_if_fail (self != NULL);
	g_return_if_fail (ccall != NULL);
	g_return_if_fail (type_args != NULL);
	g_return_if_fail (expr != NULL);
	{
		GeeIterator* _type_arg_it;
		_type_arg_it = gee_iterable_iterator ((GeeIterable*) type_args);
		while (TRUE) {
			ValaDataType* type_arg;
			if (!gee_iterator_next (_type_arg_it)) {
				break;
			}
			type_arg = (ValaDataType*) gee_iterator_get (_type_arg_it);
			if (VALA_IS_GENERIC_TYPE (type_arg)) {
				char* type_param;
				ValaCCodeIdentifier* _tmp1_;
				char* _tmp0_;
				ValaCCodeIdentifier* _tmp3_;
				char* _tmp2_;
				ValaCCodeIdentifier* _tmp5_;
				char* _tmp4_;
				/* map generic type parameter*/
				type_param = g_utf8_strdown (vala_symbol_get_name ((ValaSymbol*) vala_data_type_get_type_parameter (type_arg)), -1);
				_tmp1_ = NULL;
				_tmp0_ = NULL;
				vala_ccode_function_call_add_argument (ccall, (ValaCCodeExpression*) (_tmp1_ = vala_ccode_identifier_new (_tmp0_ = g_strdup_printf ("%s_type", type_param))));
				(_tmp1_ == NULL) ? NULL : (_tmp1_ = (vala_ccode_node_unref (_tmp1_), NULL));
				_tmp0_ = (g_free (_tmp0_), NULL);
				_tmp3_ = NULL;
				_tmp2_ = NULL;
				vala_ccode_function_call_add_argument (ccall, (ValaCCodeExpression*) (_tmp3_ = vala_ccode_identifier_new (_tmp2_ = g_strdup_printf ("%s_dup_func", type_param))));
				(_tmp3_ == NULL) ? NULL : (_tmp3_ = (vala_ccode_node_unref (_tmp3_), NULL));
				_tmp2_ = (g_free (_tmp2_), NULL);
				_tmp5_ = NULL;
				_tmp4_ = NULL;
				vala_ccode_function_call_add_argument (ccall, (ValaCCodeExpression*) (_tmp5_ = vala_ccode_identifier_new (_tmp4_ = g_strdup_printf ("%s_destroy_func", type_param))));
				(_tmp5_ == NULL) ? NULL : (_tmp5_ = (vala_ccode_node_unref (_tmp5_), NULL));
				_tmp4_ = (g_free (_tmp4_), NULL);
				type_param = (g_free (type_param), NULL);
			} else {
				ValaCCodeIdentifier* _tmp7_;
				char* _tmp6_;
				_tmp7_ = NULL;
				_tmp6_ = NULL;
				vala_ccode_function_call_add_argument (ccall, (ValaCCodeExpression*) (_tmp7_ = vala_ccode_identifier_new (_tmp6_ = vala_data_type_get_type_id (type_arg))));
				(_tmp7_ == NULL) ? NULL : (_tmp7_ = (vala_ccode_node_unref (_tmp7_), NULL));
				_tmp6_ = (g_free (_tmp6_), NULL);
				if (vala_ccode_base_module_requires_copy ((ValaCCodeBaseModule*) self, type_arg)) {
					ValaCCodeExpression* dup_func;
					ValaCCodeCastExpression* _tmp8_;
					ValaCCodeExpression* _tmp9_;
					dup_func = vala_ccode_base_module_get_dup_func_expression ((ValaCCodeBaseModule*) self, type_arg, vala_code_node_get_source_reference ((ValaCodeNode*) type_arg));
					if (dup_func == NULL) {
						/* type doesn't contain a copy function*/
						vala_code_node_set_error (expr, TRUE);
						(dup_func == NULL) ? NULL : (dup_func = (vala_ccode_node_unref (dup_func), NULL));
						(type_arg == NULL) ? NULL : (type_arg = (vala_code_node_unref (type_arg), NULL));
						(_type_arg_it == NULL) ? NULL : (_type_arg_it = (gee_collection_object_unref (_type_arg_it), NULL));
						return;
					}
					_tmp8_ = NULL;
					vala_ccode_function_call_add_argument (ccall, (ValaCCodeExpression*) (_tmp8_ = vala_ccode_cast_expression_new (dup_func, "GBoxedCopyFunc")));
					(_tmp8_ == NULL) ? NULL : (_tmp8_ = (vala_ccode_node_unref (_tmp8_), NULL));
					_tmp9_ = NULL;
					vala_ccode_function_call_add_argument (ccall, _tmp9_ = vala_ccode_base_module_get_destroy_func_expression ((ValaCCodeBaseModule*) self, type_arg));
					(_tmp9_ == NULL) ? NULL : (_tmp9_ = (vala_ccode_node_unref (_tmp9_), NULL));
					(dup_func == NULL) ? NULL : (dup_func = (vala_ccode_node_unref (dup_func), NULL));
				} else {
					ValaCCodeConstant* _tmp10_;
					ValaCCodeConstant* _tmp11_;
					_tmp10_ = NULL;
					vala_ccode_function_call_add_argument (ccall, (ValaCCodeExpression*) (_tmp10_ = vala_ccode_constant_new ("NULL")));
					(_tmp10_ == NULL) ? NULL : (_tmp10_ = (vala_ccode_node_unref (_tmp10_), NULL));
					_tmp11_ = NULL;
					vala_ccode_function_call_add_argument (ccall, (ValaCCodeExpression*) (_tmp11_ = vala_ccode_constant_new ("NULL")));
					(_tmp11_ == NULL) ? NULL : (_tmp11_ = (vala_ccode_node_unref (_tmp11_), NULL));
				}
			}
			(type_arg == NULL) ? NULL : (type_arg = (vala_code_node_unref (type_arg), NULL));
		}
		(_type_arg_it == NULL) ? NULL : (_type_arg_it = (gee_collection_object_unref (_type_arg_it), NULL));
	}
}


static void vala_ccode_method_call_module_class_init (ValaCCodeMethodCallModuleClass * klass) {
	vala_ccode_method_call_module_parent_class = g_type_class_peek_parent (klass);
	VALA_CCODE_MODULE_CLASS (klass)->visit_method_call = vala_ccode_method_call_module_real_visit_method_call;
}


static void vala_ccode_method_call_module_instance_init (ValaCCodeMethodCallModule * self) {
}


GType vala_ccode_method_call_module_get_type (void) {
	static GType vala_ccode_method_call_module_type_id = 0;
	if (vala_ccode_method_call_module_type_id == 0) {
		static const GTypeInfo g_define_type_info = { sizeof (ValaCCodeMethodCallModuleClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) vala_ccode_method_call_module_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (ValaCCodeMethodCallModule), 0, (GInstanceInitFunc) vala_ccode_method_call_module_instance_init, NULL };
		vala_ccode_method_call_module_type_id = g_type_register_static (VALA_TYPE_CCODE_ASSIGNMENT_MODULE, "ValaCCodeMethodCallModule", &g_define_type_info, 0);
	}
	return vala_ccode_method_call_module_type_id;
}


static int _vala_strcmp0 (const char * str1, const char * str2) {
	if (str1 == NULL) {
		return -(str1 != str2);
	}
	if (str2 == NULL) {
		return str1 != str2;
	}
	return strcmp (str1, str2);
}




