/* valamethodtype.vala
 *
 * Copyright (C) 2007-2008  Jürg Billeter
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.

 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.

 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA
 *
 * Author:
 * 	Jürg Billeter <j@bitron.ch>
 */

using GLib;
using Gee;

/**
 * The type of a method referencea.
 */
public class Vala.MethodType : DataType {
	public Method method_symbol { get; set; }

	public MethodType (Method method_symbol) {
		this.method_symbol = method_symbol;
	}

	public override bool is_invokable () {
		return true;
	}

	public override DataType? get_return_type () {
		return method_symbol.return_type;
	}

	public override Gee.List<FormalParameter>? get_parameters () {
		return method_symbol.get_parameters ();
	}

	public override DataType copy () {
		return new MethodType (method_symbol);
	}

	public override bool compatible (DataType target_type) {
		var dt = target_type as DelegateType;
		if (dt == null) {
			// method types incompatible to anything but delegates
			return false;
		}
		
		return dt.delegate_symbol.matches_method (method_symbol);
	}

	public override string to_qualified_string (Scope? scope) {
		return method_symbol.get_full_name ();
	}

	public override string? get_cname () {
		return "gpointer";
	}

	public override Symbol? get_member (string member_name) {
		if (method_symbol.coroutine && (member_name == "begin" || member_name == "end")) {
			return method_symbol;
		}
		return null;
	}

	public Gee.List<FormalParameter> get_member_parameters (string? member_name) {
		Gee.List<FormalParameter> params = method_symbol.get_parameters ();
		if (method_symbol.coroutine) {
			var root_symbol = method_symbol.source_reference.file.context.root as Symbol;
			var glib_symbol = root_symbol.scope.lookup ("GLib");

			Gee.List<FormalParameter> async_params = new Gee.ArrayList<FormalParameter> (direct_equal);
			if (member_name == "begin") {
				foreach (FormalParameter param in params) {
					if (param.direction == ParameterDirection.IN) {
						async_params.add (param);
					}
				}

				var callback_type = new DelegateType ((Delegate) glib_symbol.scope.lookup ("AsyncReadyCallback"));
				callback_type.nullable = true;

				var callback_param = new FormalParameter ("callback", callback_type);
				callback_param.default_expression = new NullLiteral ();
				callback_param.cparameter_position = async_params.size + 1;
				callback_param.cdelegate_target_parameter_position = callback_param.cparameter_position + 0.1;

				async_params.add (callback_param);
			} else if (member_name == "end") {
				foreach (FormalParameter param in params) {
					if (param.direction != ParameterDirection.IN) {
						async_params.add (param);
					}
				}

				var result_type = new ObjectType ((ObjectTypeSymbol) glib_symbol.scope.lookup ("AsyncResult"));

				var result_param = new FormalParameter ("async_result", result_type);
				result_param.cparameter_position = -1.1;

				async_params.add (result_param);
			} else {
				return params;
			}
			return async_params;
		}
		return params;
	}

	public DataType get_member_return_type (string? member_name) {
		if (method_symbol.coroutine) {
			if (member_name == "begin") {
				return new VoidType ();
			} else if (member_name == "end") {
				return method_symbol.return_type;
			}
		}
		return method_symbol.return_type;
	}
}
